<?php

namespace App\Http\Controllers\Admin;

use Carbon\Carbon;
use App\Models\User;
use App\Models\Pages;
use App\Models\AddSpace;
use App\Models\Apptitle;
use App\Models\Customer;
use App\Models\Translate;
use App\Models\Footertext;
use App\Models\Seosetting;
use Jenssegers\Agent\Agent;
use App\Models\Imap_setting;
use Illuminate\Http\Request;
use Torann\GeoIP\Facades\GeoIP;
use App\Models\EmployeeActivity;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Validator;


class AddSpaceController extends Controller
{
    public function index()
    {

        $this->authorize('Addspace Access');
        $basic = Apptitle::first();

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $addspace = AddSpace::all();
        $data['addspace'] = $addspace;

        return view('admin.addspace.index')->with($data);
    }


    public function storeAddSpace(Request $req)
    {

        $rules = [
            'addSpaceTitle' => 'required|string|max:255|unique:add_spaces,title,' . $req->id,
            'addSpaceLink' => 'required|string',
        ];

        if (!$req->id) {
            $rules['addspaceImage'] = 'required|mimes:jpg,jpeg,png,svg|max:5120';
        } else {
            $rules['addspaceImage'] = 'nullable|mimes:jpg,jpeg,png,svg|max:5120';
        }

        // Validate the request with the defined rules
        $validator = Validator::make($req->all(), $rules);

        // Check if validation fails
        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $addSpaceImage = null;

        // Handle file upload
        if ($req->hasFile('addspaceImage')) {
            $file = $req->file('addspaceImage');

            if ($file->isValid()) {
                $destinationPath = public_path('/uploads/addspace/');
                if (!File::exists($destinationPath)) {
                    File::makeDirectory($destinationPath, 0777, true);
                }
                $addSpaceImage = date('YmdHis') . "." . $file->getClientOriginalExtension();
                $file->move($destinationPath, $addSpaceImage);
            } else {
                return redirect()->back()->with('error', lang('The uploaded file is invalid or empty.', 'alerts'));
                // return response()->json(['error' => lang('.', 'alerts')]);
            }
        }

        // Use updateOrCreate
        $addspace = AddSpace::updateOrCreate(
            ['id' => $req->id],
            [
                'title' => $req->addSpaceTitle,
                'status' => $req->status == 'on' ? 1 : 0,
                'link' => $req->addSpaceLink,
                'image' => $addSpaceImage ?? ($req->id ? DB::table('add_spaces')->where('id', $req->id)->value('image') : null),
            ]
        );

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = $req->id ? 'Updated an addspace' . ' ' . $req->addSpaceTitle : 'created an addspace' . $req->addSpaceTitle;
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        $message = $req->id ? lang('Addspace updated successfully.', 'alerts') : lang('Addspace created successfully.', 'alerts');

        // Return success response
        return response()->json(['success' => $message, 'data' => $addspace]);
    }

    public function deleteAddSpace($id)
    {
        $this->authorize('Addspace Delete');
        $AddSpace = AddSpace::find($id);
        $AddSpace->delete();

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted an addspace' . ' ' . $AddSpace->title;
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return response()->json(['success' => lang('The AddSpace has been successfully deleted.', 'alerts')]);
    }

    public function deleteall(Request $request)
    {
        $this->authorize('Addspace Delete');
        $id_array = $request->input('id');

        $Addspace = AddSpace::whereIn('id', $id_array)->get();

        foreach ($Addspace as $add) {
            $add->delete();
        }

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted multiple addspace' . ' ';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return response()->json(['success' => lang('The AddSpace has been successfully deleted.', 'alerts')]);
    }

    public function changeStatus(Request $request, $id)
    {
        try {
            $this->authorize('AddSpace Edit');
            $supID = AddSpace::find($id);
            $supID->status = $request->status;
            $supID->save();

            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $agent = new Agent();
            $activity = new EmployeeActivity();
            $activity->user_id = Auth::user()->id;
            $activity->activity_type = 'Changed Addspace Status' . ' ' . $supID->title;
            $activity->ip_address = $geolocation->ip;
            $activity->browser = $agent->browser();
            $activity->device = $agent->device();
            $activity->save();

            return response()->json(['code' => 200, 'success' => lang('Updated successfully', 'alerts')], 200);
        } catch (\Throwable $th) {
            return response()->json(['exception' => $th->getMessage()]);
        }
    }
}
