<?php

namespace App\Http\Controllers\Admin;

use DB;
use DataTables;

use Carbon\Carbon;
use App\Models\User;
use Hashids\Hashids;
use App\Models\Pages;
use App\Models\Apptitle;
use App\Models\Customer;
use App\Models\Footertext;
use App\Models\Seosetting;
use App\Models\Userrating;
use App\Models\usersettings;
use Illuminate\Http\Request;
use App\Models\Ticket\Ticket;
use App\Models\Employeerating;
use App\Models\Ticket\Comment;
use App\Models\Articles\Article;
use App\Models\EmployeeActivity;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Crypt;

class AdminReportController extends Controller
{
   public function index()
   {


      $this->authorize('Reports Access');

      $users = User::latest('updated_at')->paginate(6);
      $data['users'] = $users;

      $title = Apptitle::first();
      $data['title'] = $title;

      $footertext = Footertext::first();
      $data['footertext'] = $footertext;

      $seopage = Seosetting::first();
      $data['seopage'] = $seopage;

      $post = Pages::all();
      $data['page'] = $post;

      $agentactivec = User::where('status', '1')->count();
      $data['agentactivec'] = $agentactivec;
      $agentinactive = User::where('status', '0')->count();
      $data['agentinactive'] = $agentinactive;

      $customeractive = Customer::where('status', '1')->count();
      $data['customeractive'] = $customeractive;
      $customerinactive = Customer::where('status', '0')->count();
      $data['customerinactive'] = $customerinactive;

      $newticket = Ticket::where('status', 'New')->count();
      $data['newticket'] = $newticket;

      $closedticket = Ticket::where('status', 'Closed')->count();
      $data['closedticket'] = $closedticket;

      $inprogressticket = Ticket::where('status', 'Inprogress')->count();
      $data['inprogressticket'] = $inprogressticket;

      $onholdticket = Ticket::where('status', 'On-Hold')->count();
      $data['onholdticket'] = $onholdticket;

      $reopenticket = Ticket::where('status', 'Re-Open')->count();
      $data['reopenticket'] = $reopenticket;

      $prioritylow = Ticket::where('priority', 'Low')->count();
      $data['prioritylow'] = $prioritylow;

      $priorityhigh = Ticket::where('priority', 'High')->count();
      $data['priorityhigh'] = $priorityhigh;

      $prioritymedium = Ticket::where('priority', 'Medium')->count();
      $data['prioritymedium'] = $prioritymedium;

      $prioritycritical = Ticket::where('priority', 'Critical')->count();
      $data['prioritycritical'] = $prioritycritical;

      $articlepublished = Article::where('status', 'Published')->count();
      $data['articlepublished'] = $articlepublished;

      $articleunpublished = Article::where('status', 'UnPublished')->count();
      $data['articleunpublished'] = $articleunpublished;

      $activity = EmployeeActivity::latest('created_at')->paginate(6);
      $data['activity'] = $activity;


      return view('admin.reports.index')->with($data);
   }

   public function ticketreports()
   {

      $users = User::with('role')->get();

      // Prepare an array to hold user ratings
      $userRatings = [];

      foreach ($users as $user) {
         // Get counts of each rating for the current user
         $avgrating1 = Employeerating::where('user_id', $user->id)
            ->where('rating', '1')
            ->count();
         $avgrating2 = Employeerating::where('user_id', $user->id)
            ->where('rating', '2')
            ->count();
         $avgrating3 = Employeerating::where('user_id', $user->id)
            ->where('rating', '3')
            ->count();
         $avgrating4 = Employeerating::where('user_id', $user->id)
            ->where('rating', '4')
            ->count();
         $avgrating5 = Employeerating::where('user_id', $user->id)
            ->where('rating', '5')
            ->count();

         // Calculate average rating
         $avgr =
            5 * $avgrating5 +
            4 * $avgrating4 +
            3 * $avgrating3 +
            2 * $avgrating2 +
            1 * $avgrating1;
         $avggr =
            $avgrating1 + $avgrating2 + $avgrating3 + $avgrating4 + $avgrating5;

         $avg = $avggr == 0 ? 0 : $avgr / $avggr;

         // Get overall count of ratings for the user
         $overallCount = Employeerating::where('user_id', $user->id)->count();

         // Get comments related to the user
         $comenttickets = $user->comments()->whereNotNull('user_id')->distinct()->get();

         // Count distinct tickets from comments
         $comentticketss = Ticket::leftJoin(
            'comments',
            'comments.ticket_id',
            'tickets.id'
         )
            ->where('comments.user_id', $user->id)
            ->distinct('comments.ticket_id')
            ->count();

         // Store the rating, overall count, comments, and ticket counts
         $userRatings[$user->id] = [
            'user' => $user,
            'rating' => $avg,
            'overallCount' => $overallCount,
            'comentticketss' => $comentticketss,
         ];
      }
      $data['userRatings'] = $userRatings;

      $title = Apptitle::first();
      $data['title'] = $title;

      $footertext = Footertext::first();
      $data['footertext'] = $footertext;

      $seopage = Seosetting::first();
      $data['seopage'] = $seopage;

      $post = Pages::all();
      $data['page'] = $post;

      $activity = EmployeeActivity::latest('created_at')->paginate(6);
      $data['activity'] = $activity;



      return view('admin.reports.ticketratingreport')->with($data);
   }

   public function employeedetails($id)
   {
      $users = User::find($id);
      $data['users'] = $users;

      $employeerating = Ticket::select('tickets.*')->leftJoin('comments', 'comments.ticket_id', 'tickets.id')
         ->where('comments.user_id', $users->id)->distinct('comments.ticket_id', 'tickets.id')->get();

      $employeerating = $employeerating->each(function ($rating) {
         $hashids = app(Hashids::class);
         $rating->encrypted_id = $hashids->encode($rating->id);
         // $rating->encrypted_id = Crypt::encrypt($rating->id);
         return $rating;
      });

      // Process each rating to calculate average ratings
      $employeerating = $employeerating->map(function ($ticket) {
         // Calculate average rating for each ticket
         $avgrating1 = Userrating::where('ticket_id', $ticket->id)->where('ratingstar', '1')->count();
         $avgrating2 = Userrating::where('ticket_id', $ticket->id)->where('ratingstar', '2')->count();
         $avgrating3 = Userrating::where('ticket_id', $ticket->id)->where('ratingstar', '3')->count();
         $avgrating4 = Userrating::where('ticket_id', $ticket->id)->where('ratingstar', '4')->count();
         $avgrating5 = Userrating::where('ticket_id', $ticket->id)->where('ratingstar', '5')->count();

         $avgr = ((5 * $avgrating5) + (4 * $avgrating4) + (3 * $avgrating3) + (2 * $avgrating2) + (1 * $avgrating1));
         $avggr = ($avgrating1 + $avgrating2 + $avgrating3 + $avgrating4 + $avgrating5);

         // Avoid division by zero
         if ($avggr == 0) {
            $avggr = 1;
         }

         $avg = $avgr / $avggr;

         // Add calculated average rating to the ticket
         $ticket->average_rating = $avg;

         // Retrieve user ratings and comments for the ticket
         $ticket->userRatings = Userrating::with('customer')->where('ticket_id', $ticket->id)->get();

         return $ticket;
      });

      $data['employeeratings'] = $employeerating;


      $title = Apptitle::first();
      $data['title'] = $title;

      $footertext = Footertext::first();
      $data['footertext'] = $footertext;

      $seopage = Seosetting::first();
      $data['seopage'] = $seopage;

      $post = Pages::all();
      $data['page'] = $post;

      $activity = EmployeeActivity::latest('created_at')->paginate(6);
      $data['activity'] = $activity;

      return view('admin.reports.ratingview')->with($data);
   }

   public function ratingticketdelete($id)
   {
      $ticketratingdelete = Userrating::where('ticket_id', $id)->first();

      $employeeratingdelete = Employeerating::where('urating_id', $ticketratingdelete->id)->get();
      foreach ($employeeratingdelete as $employeesrating) {
         $employeesrating->delete();
      }
      $ticketratingdelete->delete();

      return response()->json(['success' => lang('Delete successfully', 'alerts')]);
   }

   public function employeeActivity()
   {
      $this->authorize('Reports Access');

      $title = Apptitle::first();
      $data['title'] = $title;

      $footertext = Footertext::first();
      $data['footertext'] = $footertext;

      $seopage = Seosetting::first();
      $data['seopage'] = $seopage;

      $activity = EmployeeActivity::latest('created_at')->paginate(15);
      $data['activity'] = $activity;

      $data['jsempactivity'] = EmployeeActivity::with('user')->get();

      return view('admin.reports.employeeActivity')->with($data);
   }
}
