<?php

namespace App\Http\Controllers\Admin\Auth;

use File;

use Mail;
use Session;
use Carbon\Carbon;

use App\Models\User;
use App\Jobs\MailSend;
use App\Models\Holiday;
use App\Models\Setting;
use App\Models\Apptitle;

use App\Models\VerifyOtp;
use App\Models\Seosetting;
use Jenssegers\Agent\Agent;
use App\Models\Announcement;
use Illuminate\Http\Request;
use App\Mail\mailmailablesend;
use Torann\GeoIP\Facades\GeoIP;
use App\Models\EmployeeActivity;
use App\Models\SocialAuthSetting;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;

use App\Providers\RouteServiceProvider;
use Spatie\SslCertificate\SslCertificate;
use Illuminate\Validation\ValidationException;
use Illuminate\Foundation\Auth\ThrottlesLogins;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use laravelLara\lskusd\utils\trait\TraitCheckHelperAPI;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */


    use ThrottlesLogins,AuthenticatesUsers {
        logout as performLogout;
    }

    use TraitCheckHelperAPI;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = RouteServiceProvider::HOME;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest')->except('logout');
    }


    public function showloginform(){

        $title = Apptitle::first();
        $data['title'] = $title;

        $socialAuthSettings = SocialAuthSetting::first();
        $data['socialAuthSettings'] = $socialAuthSettings;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $now = now();
        $announcement = announcement::whereDate('enddate', '>=', $now->toDateString())->whereDate('startdate', '<=', $now->toDateString())->get();
        $data['announcement'] = $announcement;

        $announcements = Announcement::whereNotNull('announcementday')->get();
        $data['announcements'] = $announcements;

        // $holidays = Holiday::whereDate('startdate', '<=', $now->toDateString())->whereDate('enddate', '>=', $now->toDateString())->where('status', '1')->get();
        // $data['holidays'] =  $holidays;

        $now = now();
        $currentYear = $now->year;
        $activeHolidays = Holiday::where('status', 1)->get();

       
        $data['holidays'] = $activeHolidays->filter(function ($holiday) use ($now, $currentYear) {
            $holidayStart = Carbon::parse($holiday->startdate);
            $holidayEnd = Carbon::parse($holiday->enddate);
            if ($holiday->is_fixed) {
                return $holidayStart->month === $now->month && $holidayStart->day === $now->day;
            }
            return $holidayStart->isSameDay($now);
        });


        return view('admin.auth.login', ['url'=> 'admin/login'])->with($data);
    }

    protected function credentials(Request $request)
    {
        return [
            'email' => $request->{$this->username()},
            'password' => $request->password,
            'status' => '1'
        ];
    }

    public function redirectTo(){

            $user = auth()->user();
            if ($user) {

                return 'admin/';
            }



    }
    protected function validateLogin(Request $request)
    {

        if(setting('CAPTCHATYPE') == 'off'){
            $rules = [
                'email' => 'required|string|max:255',
                'password' => 'required|string|max:255'
            ];
        }else{
            if(setting('CAPTCHATYPE') == 'manual'){
                if(setting('RECAPTCH_ENABLE_ADMIN_LOGIN')=='yes'){

                    $rules = [
                        'email' => 'required|string|max:255',
                        'password' => 'required|string|max:255',
                        'captcha' => ['required', 'captcha'],
                    ];

                }else{

                    $rules = [
                        'email' => 'required|string|max:255',
                        'password' => 'required|string|max:255'
                    ];

                }

            }
            if(setting('CAPTCHATYPE') == 'google'){
                if(setting('RECAPTCH_ENABLE_ADMIN_LOGIN')=='yes'){

                    $rules = [
                        'email' => 'required|string|max:255',
                        'password' => 'required|string|max:255',
                        'g-recaptcha-response'  =>  'required|recaptcha',
                    ];

                }else{
                    $rules = [
                        'email' => 'required|string|max:255',
                        'password' => 'required|string|max:255'
                    ];
                }
            }
        }




        // User type from email/username
        $user = User::where($this->username(), $request->{$this->username()})->first();

        $this->validate($request, $rules);
    }
    public function userInactiveMessage()
    {
        throw ValidationException::withMessages([
            $this->username() => ['error'=> lang('Your Account is Inactive. Please Contact to Admin.', 'alerts')],
        ]);
    }

    private function updateSettings($data)
    {
        foreach($data as $key => $val){
        	$setting = Setting::where('key', $key);
        	if( $setting->exists() )
        		$setting->first()->update(['value' => $val]);
        }

    }

    public function login(Request $request)
    {
        $this->validateLogin($request);
        $user = User::where($this->username(), $request->{$this->username()})->first();
        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();

        
        if ($user) {

            if ($user && $user->status == '0') {
                return $this->userInactiveMessage();
            }

            if ($user && $user->verified == '0') {
                return $this->userverifiedMessage();
            }

            // If the class is using the ThrottlesLogins trait, we can automatically throttle
            // the login attempts for this application. We'll key this by the username and
            // the IP address of the client making these requests into this application.
            if (
                method_exists($this, 'hasTooManyLoginAttempts') &&
                $this->hasTooManyLoginAttempts($request)
            ) {
                $this->fireLockoutEvent($request);

                return $this->sendLockoutResponse($request);
            }

            if ($this->attemptLogin($request)) {
                if(setting('Employe_google_two_fact') == 'on' && User::where(['id' =>$user->id, 'twofactorauth' => 'googletwofact'])->exists()){
                    $request->session()->put('google2faemail',$request->email);
                    return redirect()->route('admin.google2falogin',['email' =>$request->email]);

                }

                if(setting('Employe_email_two_fact') == 'on' && User::where(['id' =>$user->id, 'twofactorauth' => 'emailtwofact'])->exists()){
                    if($verifyuser = VerifyOtp::where('cust_id',$user->email)->exists()){
                        $verifyuser = VerifyOtp::where('cust_id',$user->email)->first();
                        $verifyuser->delete();
                    }
                    $email = $user->email;
                    $verifyOtp = VerifyOtp::create([
                        'cust_id' => $email,
                        'otp' => rand(100000, 999999),
                        'type' => 'twofactorotp',
                    ]);

                    $guestticket = [

                        'otp' => $verifyOtp->otp,
                        'guestemail' => $verifyOtp->cust_id,
                        'guestname' => 'adminuser',
                    ];
                    try {

                        dispatch((new MailSend($verifyOtp->cust_id, 'two_factor_authentication_otp_send', $guestticket)));

                    } catch (\Exception$e) {

                    }
                    return redirect()->route('admin.emailtwofactorlogin',$email);
                }

                $activity = new EmployeeActivity();
                $activity->user_id = $user->id;
                $activity->activity_type = 'Logged in to the application';
                $activity->ip_address = $geolocation->ip;
                $activity->browser = $agent->browser();
                $activity->device = $agent->device();
                $activity->save();

                return $this->sendLoginResponse($request);
            }
        }
        else {
            throw ValidationException::withMessages([
                $this->username() => ['error'=> lang('Account with email does not exist.', 'alerts')],
            ]);
        }

        // If the login attempt was unsuccessful we will increment the number of attempts
        // to login and redirect the user back to the login form. Of course, when this
        // user surpasses their maximum number of attempts they will get locked out.
        $this->incrementLoginAttempts($request);

        $activity = new EmployeeActivity();
        $activity->user_id = $user->id;
        $activity->activity_type = 'Failed login attempt';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return $this->sendFailedLoginResponse($request);
    }

    public function validatelog()
    {
        $sessionPath = storage_path('framework/sessions');
        $sessionFiles = glob($sessionPath . '/*');
        foreach ($sessionFiles as $sessionFile) {
            unlink($sessionFile);
        }
        Session::flush();

        return redirect()->route('login');
    }

    public function userverifiedMessage()
    {
        throw ValidationException::withMessages([
            $this->username() => ['error'=> lang('Your Account is Not Verified.', 'alerts')],
        ]);
    }

    public function logout(Request $request)
    {

        $authId = auth()->user() ? auth()->user()->id : null; 
        if (!$authId) {
            return response()->json(['error' => 'User not found']);
        }
        $this->performLogout($request);

        // $authId = auth()->user()->id;
        // $this->performLogout($request);

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = $authId;
        $activity->activity_type = 'Logged out of the application';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return redirect()->route('login');
    }
}
