<?php

namespace App\Http\Controllers\Admin;

use App\Models\Pages;
use App\Models\Apptitle;

use App\Models\Footertext;
use App\Models\Seosetting;
use App\Models\Customfield;
use Jenssegers\Agent\Agent;
use Illuminate\Http\Request;
use Torann\GeoIP\Facades\GeoIP;
use App\Models\EmployeeActivity;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;

class CustomfieldController extends Controller
{
    public function index()
    {

        $this->authorize('CustomField Access');

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $customfields = Customfield::all();
        $data['customfields'] = $customfields;

        return view('admin.customfield.index')->with($data);

    }

    public function storeupdate(Request $request)
    {
        $sprukofieldtype = $request->sprukofieldtype;

        if ($sprukofieldtype == 'select' || $sprukofieldtype == 'checkbox' || $sprukofieldtype == 'radio') {
            $required = 'required';
        } else {
            $required = '';
        }

        $request->validate(
            [
            'sprukofieldname'=> 'required',
            'display'=> 'required',
            'optionsfields' => $required,
            // 'optionsfields' => 'required_if:sprukofieldtype,select|required_if:sprukofieldtype,checkbox|required_if:sprukofieldtype,radio',
            ],
            [

            'sprukofieldname.required'=> 'Your Label Field Name is Required',
            'display.required' => 'Your View On Field is Required',
            'optionsfields.required' => 'Options field is required when field type is select, checkbox, or radio',
            ]

        );


        $customfieldId = $request->customfieldopen_id;
        $customfield =  [
            'fieldtypes' => $request->sprukofieldtype,
            'fieldnames' => $request->sprukofieldname,
            'fieldprivacy' => $request->privacyfields,
            'fieldrequired' => $request->requiredfields,
            'status' => $request->status,
            'fieldoptions' => $request->optionsfields,
            'displaytypes' => $request->display,

        ];



        $custom = Customfield::updateOrCreate(['id' => $customfieldId], $customfield);

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = $customfieldId ? 'updated a custom field' : 'Created a custom field';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        $message = $customfieldId ? lang('Customfield updated successfully.', 'alerts') : lang('Customfield created successfully.', 'alerts');

        return response()->json(['success' => $message,'data'=>$custom], 200);
    }

    public function edit($id)
    {
        $this->authorize('CustomField Edit');

        $customfieldfind = Customfield::findOrFail($id);

        return response()->json($customfieldfind, 200);
    }

    public function destroy($id)
    {
        $this->authorize('CustomField Delete');

        $customfielddelete = Customfield::findOrFail($id);
        $customfielddelete->delete();

            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $agent = new Agent();
            $activity = new EmployeeActivity();
            $activity->user_id = Auth::user()->id;
            $activity->activity_type = 'Deleted a custom field';
            $activity->ip_address = $geolocation->ip;
            $activity->browser = $agent->browser();
            $activity->device = $agent->device();
            $activity->save();
        return response()->json(['success' => lang('Deleted successfully', 'alerts')], 200);
    }

    public function destroyall(Request $request)
    {
        $this->authorize('CustomField Delete');

        $id_array = $request->input('id');

        $customfields = Customfield::whereIn('id', $id_array)->get();

        foreach($customfields as $customfield)
        {
            $customfield->delete();

        }
            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $agent = new Agent();
            $activity = new EmployeeActivity();
            $activity->user_id = Auth::user()->id;
            $activity->activity_type = 'Deleted custom fields';
            $activity->ip_address = $geolocation->ip;
            $activity->browser = $agent->browser();
            $activity->device = $agent->device();
            $activity->save();
        return response()->json(['success'=> lang('Deleted successfully', 'alerts')]);
    }

    public function status(Request $request, $id)
    {
        $this->authorize('CustomField Edit');

        $customfieldfind = Customfield::find($id);
        $customfieldfind->status = $request->status;
        $customfieldfind->save();

            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $agent = new Agent();
            $activity = new EmployeeActivity();
            $activity->user_id = Auth::user()->id;
            $activity->activity_type = 'Changed status of custom field';
            $activity->ip_address = $geolocation->ip;
            $activity->browser = $agent->browser();
            $activity->device = $agent->device();
            $activity->save();

        return response()->json(['code'=>200, 'success'=>lang('Updated Successfully', 'alerts')], 200);

    }
}
