<?php

namespace App\Http\Controllers\Admin;

use DB;
use Str;

use DataTables;
use App\Models\User;
use App\Models\Pages;
use App\Models\Groups;
use App\Models\Apptitle;
use App\Models\Footertext;
use App\Models\Seosetting;
use Jenssegers\Agent\Agent;
use App\Models\CategoryUser;
use Illuminate\Http\Request;
use Torann\GeoIP\Facades\GeoIP;
use App\Models\EmployeeActivity;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;


class GroupCreateController extends Controller
{
    public function index()
    {

        $this->authorize('Groups List Access');
        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $groups = Groups::with('groupsuser')->get();
        $data['groups'] = $groups;

        return view('admin.groups.index')->with($data);
    }

    public function create()
    {
        $this->authorize('Groups Create');
        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $users = User::where('status', 1)->get();
        $data['users'] = $users;


        return view('admin.groups.create')->with($data);
    }


    public function store(Request $request)
    {
        $this->authorize('Groups Create');
        $request->validate([
            'groupname' => 'required|string|max:255|unique:groups',
            'user_id' => 'required',

        ]);
        $grop = new Groups;

        $grop->groupname = $request->input('groupname');
        $grop->groupstatus = 0;
        $grop->save();

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Created a group';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        if ($request->input('user_id')) {
            foreach ($request->input('user_id') as $value) {
                $user_id[] = $value;
            }
        }

        $grop->groupsusers()->sync($request->get('user_id'));

        return redirect('admin/groups')->with('success', lang('A group was successfully created.', 'alerts'));
    }

    public function show($id)
    {
        $this->authorize('Groups Edit');
        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $grop = Groups::find($id);
        $data['group'] = $grop;

        $group = DB::table("groups_users")->where("groups_users.groups_id", $id)
            ->pluck('groups_users.users_id', 'groups_users.users_id')
            ->all();
        $data['grop'] = $group;

        $users = User::where('status', '1')->get();
        $data['users'] = $users;

        return view('admin.groups.edit')->with($data);
    }

    public function update(Request $request, $id)
    {

        $this->authorize('Groups Edit');
        $request->validate([
            'groupname' => 'required|string|max:255',
            'user_id' => 'required',

        ]);
        $grop = Groups::find($id);
        if ($grop->groupname == $request->groupname) {

            $grop->update();
            if ($request->input('user_id')) {
                foreach ($request->input('user_id') as $value) {
                    $user_id[] = $value;
                }
            }

            $grop->groupsusers()->sync($request->get('user_id'));
        } else {
            $request->validate([
                'groupname' => 'required|string|max:255|unique:groups',
            ]);

            $grop->groupname = $request->input('groupname');
            $grop->update();

            if ($request->input('user_id')) {
                foreach ($request->input('user_id') as $value) {
                    $user_id[] = $value;
                }
            }
            $grop->groupsusers()->sync($request->get('user_id'));
        }

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Updated the group';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return redirect('admin/groups')->with('success', lang('The group updated successfully.', 'alerts'));
    }

    public function destroy($id)
    {
        $groupdelete = Groups::find($id);
        $groupdelete->delete();

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted a group';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return response()->json(['success' => lang('The group deleted successfully.', 'alerts')]);
    }


    public function destroyall(Request $request)
    {
        $id_array = $request->input('id');

        $groups = Groups::whereIn('id', $id_array)->get();

        foreach ($groups as $group) {
            $group->delete();
        }

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted multiple groups';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();
        return response()->json(['success' => lang('The group deleted successfully.', 'alerts')]);
    }
}
