<?php

namespace App\Http\Controllers\Admin;

use Carbon\Carbon;
use App\Models\Pages;
use App\Models\Holiday;
use App\Models\Apptitle;
use App\Models\Footertext;
use App\Models\Seosetting;
use Jenssegers\Agent\Agent;
use App\Models\Announcement;
use Illuminate\Http\Request;
use App\Imports\HolidaysImport;
use Torann\GeoIP\Facades\GeoIP;
use App\Models\EmployeeActivity;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Validator;

class HolidayController extends Controller
{
    public function index()
    {

        $this->authorize('Holidays Access');
        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();  
        $data['seopage'] = $seopage;

        $currentYear = date('Y'); // Get the current year dynamically
        $data['holidays'] = Holiday::all()->filter(function ($holiday) use ($currentYear) {
            if ($holiday->is_fixed) {
                // For fixed holidays, always include them
                return true;
            }
            // For non-fixed holidays, check the year of the start date
            $holidayYear = Carbon::parse($holiday->startdate)->year;
                return $holidayYear >= (int)$currentYear; // Include only if the year matches
            })->map(function ($holiday) use ($currentYear) {
                // Modify the date for fixed holidays
                if ($holiday->is_fixed) {
                    $holiday->start_date = Carbon::parse($holiday->startdate)->setYear($currentYear)->format('Y-m-d');
                    $holiday->end_date = Carbon::parse($holiday->enddate)->setYear($currentYear)->format('Y-m-d');
                }
                return $holiday;
        });


        return view('admin.holidays.holidays')->with($data);
    }

    public function saveholidays(Request $request)
    {
       
        $this->authorize('Holidays Create');

        $request->validate([
            'occasion' => 'required',
            'startdate' => 'required',
            'enddate' => 'required',
            'holidaydescription' => 'required',
            'primary_color' => 'required',
            'secondary_color' => 'required',
        ]);

        $hodidayid = $request->holiday_id;
        $holidaydata =  [
            'occasion' => $request->occasion,
            'startdate' => $request->startdate,
            'enddate' => $request->enddate,
            'holidaydescription' => $request->holidaydescription,
            'primaray_color' => $request->primary_color,
            'secondary_color' => $request->secondary_color,
            'status' => $request->status,
            'is_fixed' => $request->is_fixed,

        ];

        $ipdtaa = Holiday::updateOrCreate(['id' => $hodidayid], $holidaydata);
        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = $hodidayid ? 'Updated Holidays' : 'created Holidays';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        // Set the message based on the presence of $holidayId
        $message = $hodidayid ? lang('Holidays updated successfully.', 'alerts') : lang('Holidays created successfully.', 'alerts');
        return response()->json(['success' => $message, 'data' => $ipdtaa, ]);
    }
    public function edit($id)
    {
        $this->authorize('Holidays Edit');

        $data = Holiday::find($id);
        return response()->json($data);
    }

    public function delete($id)
    {
        $this->authorize('Holidays Delete');

        $data = Holiday::find($id);
        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted a holiday';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();
        $data->delete();
    }

    public function statuschange(Request $request, $id)
    {
        $this->authorize('Holidays Edit');

        $custstatus = Holiday::find($id);
        $custstatus->status = $request->status;
        $custstatus->save();
        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Changed status of holiday';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return response()->json(['code' => 200, 'success' => lang('Status Updated successfully', 'alerts')], 200);
    }

    public function massdelete(Request $request)
    {
        $this->authorize('Holidays Delete');

        $holiday_array = $request->input('id');


        $holidays = Holiday::whereIn('id', $holiday_array)->get();

        foreach ($holidays as $holiday) {
            $holiday->delete();
        }
        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted multiple holidays';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();
        return response()->json(['success' => lang('The holidays was deleted successfully.', 'alerts')]);
    }

    public function holidaysimport()
    {
        $this->authorize('Holidays Import Access');
        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;


        return view('admin.holidays.holidaysimport')->with($data);
    }

    public function holidayscsv(Request $request)
    {
        $this->authorize('Holidays Import Access');

        if ($request->hasFile('file')) {
            $validator = Validator::make(
                $request->all(),
                [

                    'file' => 'required|file|mimes:xlsx,csv,txt',
                ],
                [
                    'file.mimes' => 'The file must be a file of type: xlsx, csv.',
                ]
            );
            if ($validator->fails()) {
                return redirect()->back()->withErrors($validator)->withInput();
            }


            $file = $request->file('file')->store('import');
            $import = new HolidaysImport;
            Excel::import($import, $file);

            $errors = $import->getErrors();

            if (!empty($errors)) {
                return redirect()->back()->withErrors($errors)->withInput();
            }
           
            // $import = Excel::import(new HolidaysImport, $file);
            if ($import->hasData) {
                $geolocation = GeoIP::getLocation(request()->getClientIp());
                $agent = new Agent();
                $activity = new EmployeeActivity();
                $activity->user_id = Auth::user()->id;
                $activity->activity_type = 'Imported holidays list';
                $activity->ip_address = $geolocation->ip;
                $activity->browser = $agent->browser();
                $activity->device = $agent->device();
                $activity->save();
                return redirect()->route('admin.holidays')->with('success', lang('The Holidays list was imported successfully.', 'alerts'));
            } else {
                return redirect()->back()->with('error', 'The imported file does not contain any data.');
            }
            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $agent = new Agent();
            $activity = new EmployeeActivity();
            $activity->user_id = Auth::user()->id;
            $activity->activity_type = 'Imported holidays list';
            $activity->ip_address = $geolocation->ip;
            $activity->browser = $agent->browser();
            $activity->device = $agent->device();
            $activity->save();
            return redirect()->route('admin.holidays')->with('success', lang('The Holidays list was imported successfully.', 'alerts'));
        } else {
            return redirect()->back()->with('error', 'Please select file to import data of Holidays.');
        }
    }
}
