<?php

namespace App\Http\Controllers\Admin;

use Str;
use Mail;
use Session;

use DataTables;
use App\Models\User;
use App\Jobs\MailSend;
use App\Mail\AppMailer;
use App\Models\Apptitle;
use App\Models\Customer;
use App\Models\Sendmail;
use App\Models\Footertext;
use App\Models\Seosetting;
use Jenssegers\Agent\Agent;
use App\Models\senduserlist;
use Illuminate\Http\Request;
use App\Mail\mailmailablesend;
use App\Jobs\BulkMails;

use Torann\GeoIP\Facades\GeoIP;
use App\Models\EmployeeActivity;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use App\Notifications\CustomerCustomNotifications;


class MailboxController extends Controller
{
    public function index()
    {
       
        $this->authorize('Custom Notifications Access');

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $customnotify = Sendmail::with(['touser.touser', 'touser.tocust', 'touser.touser.roles'])->latest()->get();
        $data['customnotify'] = $customnotify;

        return view('admin.custom-notification.index')->with($data);
    }

    public function customercompose()
    {
        $this->authorize('Custom Notifications Customer');
        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $user = Customer::where('userType', 'Customer')->get();
        $data['users'] = $user;

        return view('admin.custom-notification.customercompose')->with($data);
    }

    public function customercomposesend(Request $request)
    {
        
        $this->authorize('Custom Notifications Customer');
        $request->validate([
            'message' => 'required|max:60000',
            'subject' => 'required|max:255',
            'users' => 'required',
            'tag' => 'required|max:255',
            'selecttagcolor' => 'required',
        ]);
        $mailsend = new Sendmail();
        $mailsend->user_id = Auth::id();
        $mailsend->mailsubject = trim($request->input('subject'));
        $mailsend->mailtext = $request->input('message');
        $mailsend->tag = $request->input('tag');
        $mailsend->selecttagcolor = $request->input('selecttagcolor');
        $mailsend->save();

        foreach ($request->users as $value) {
            senduserlist::create([
                'mail_id' => $mailsend->id,
                'tocust_id' => $value,
            ]);
        }

        $cust = Customer::find($request->users);

        foreach ($cust as $value) {

            $value->notify(new CustomerCustomNotifications($mailsend));
        }

        $ticketData = [
            'notification_subject' => $mailsend->mailsubject,
            'notification_message' => $mailsend->mailtext,
            'notification_tag' => $mailsend->tag,
        ];

        // if ($request->sendType == 'sendMail') {
            try {
                $batchSize = $request->batchSize ?? 50;
                $delayBetweenBatches = $request->batchDelay ?? 5;
                $chunks = $cust->chunk($batchSize);
                foreach ($chunks as $batchIndex => $batch) {
                    $batchDelay = $batchIndex * $delayBetweenBatches;
                    $emails = $batch->pluck('email')->toArray();
                    dispatch((new BulkMails($emails, $mailsend))->delay(now()->addMinutes($batchDelay)));
                }
            } catch (\Throwable $th) {
              
                return redirect('admin/customnotification')->with('error', lang('An error occurred while sending emails.', 'alerts'));
            }

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Sends a custom notification to customer';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();
        return redirect('admin/customnotification')->with('success', lang('A custom notification was successfully sent to the customer.', 'alerts'));
    }

    public function employeecompose()
    {
        $this->authorize('Custom Notifications Employee');
        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $user = User::where('status', 1)->get();
        $data['users'] = $user;

        return view('admin.custom-notification.employeecompose')->with($data);
    }


    public function employeecomposesend(Request $request)
    {
        $this->authorize('Custom Notifications Employee');
        $request->validate([
            'message' => 'required|max:60000',
            'subject' => 'required|max:255',
            'users' => 'required',
            'tag' => 'required|max:255',
            'selecttagcolor' => 'required',
        ]);
        $mailsend = new Sendmail();
        $mailsend->user_id = Auth::id();
        $mailsend->mailsubject = $request->input('subject');
        $mailsend->mailtext = $request->input('message');
        $mailsend->tag = $request->input('tag');
        $mailsend->selecttagcolor = $request->input('selecttagcolor');
        $mailsend->save();

        foreach ($request->users as $value) {
            senduserlist::create([
                'mail_id' => $mailsend->id,
                'touser_id' => $value,
            ]);
        }
        $cust = User::find($request->users);
        foreach ($cust as $value) {

            $value->notify(new CustomerCustomNotifications($mailsend));
        }


        $ticketData = [
            'notification_subject' => $mailsend->mailsubject,
            'notification_message' => $mailsend->mailtext,
            'notification_tag' => $mailsend->tag,
        ];
        foreach ($cust as $value) {
            try {

                if ($value->usetting->emailnotifyon == 1) {
                    dispatch((new MailSend($value->email, 'when_send_customnotify_email_to_selected_member', $ticketData)));
                }
            } catch (\Exception $e) {
                $geolocation = GeoIP::getLocation(request()->getClientIp());
                $agent = new Agent();
                $activity = new EmployeeActivity();
                $activity->user_id = Auth::user()->id;
                $activity->activity_type = 'Sends a custom notification to employee';
                $activity->ip_address = $geolocation->ip;
                $activity->browser = $agent->browser();
                $activity->device = $agent->device();
                $activity->save();
                return redirect('admin/customnotification')->with('success', lang('A custom notification was successfully sent to the employee.', 'alerts'));
            }
        }

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Sends a custom notification to employee';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return redirect('admin/customnotification')->with('success', lang('A custom notification was successfully sent to the employee.', 'alerts'));
    }

    public function mailsent()
    {

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $sendmails = Sendmail::latest()->get();
        $data['sendmails'] = $sendmails;

        return view('admin.custom-notification.sendmail')->with($data);
    }

    public function show($id)
    {
        $this->authorize('Custom Notifications View');
        $custom = Sendmail::find($id);

        return response()->json($custom);
    }

    public function destroy($id)
    {
        $this->authorize('Custom Notifications Delete');
        $customdelete = Sendmail::find($id);
        $customdelete->touser()->delete();
        $customdelete->delete();

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted a custom notification';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return response()->json(['success' => lang('"Custom notification" was successfully deleted.', 'alerts')]);
    }

    public function allnotifydelete(Request $request)
    {
        $id_array = $request->input('id');

        $sendmails = Sendmail::whereIn('id', $id_array)->get();

        foreach ($sendmails as $sendmail) {
            $sendmail->touser()->delete();
            $sendmail->delete();
        }

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted multiple custom notifications';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();
        return response()->json(['success' => lang('"Custom notification" was successfully deleted.', 'alerts')]);
    }
}
