<?php

namespace App\Http\Controllers\Admin;

use DB;
use Str;
use Response;
use DataTables;
use App\Models\Pages;
use App\Models\Apptitle;
use App\Models\Projects;
use App\Models\Footertext;
use App\Models\Seosetting;
use Jenssegers\Agent\Agent;
use Illuminate\Http\Request;
use App\Imports\ProjectImport;
use App\Models\Ticket\Category;
use Torann\GeoIP\Facades\GeoIP;
use App\Models\EmployeeActivity;
use App\Models\Projects_category;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Validator;

class ProjectsController extends Controller
{
    public function index()
    {
        $this->authorize('Project Access');
        $projectss = Projects::latest()->get();
        $data['projectss'] = $projectss;

        $basic = Apptitle::first();

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $projects = Projects::all();
        $data['project'] = $projects;

        $categories = Category::whereIn('display', ['ticket', 'all'])->where('status', '1')
            ->get();
        $data['categories'] = $categories;

        $check_category = Projects_category::pluck('category_id')->toArray();
        $data['check_category'] = $check_category;

        return view('admin.projects.index', compact('basic', 'title', 'footertext'))->with($data);
    }


    public function store(Request $request)
    {
        $this->authorize('Project Create');
        $testiId = $request->projects_id;
        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',

        ]);

        if ($testiId) {
            $project = Projects::find($testiId);
            if ($project->name != $request->name) {
                $validator = Validator::make($request->all(), [
                    'name' => 'required|string|max:255|unique:projects',

                ]);
            }
        } else {
            $validator = Validator::make($request->all(), [
                'name' => 'required|string|max:255|unique:projects',

            ]);
        }


        if ($validator->passes()) {
            $testiId = $request->projects_id;


            $testi =  [
                'name' => $request->name,
            ];

            $project = Projects::updateOrCreate(['id' => $testiId], $testi);

            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $agent = new Agent();
            $activity = new EmployeeActivity();
            $activity->user_id = Auth::user()->id;
            $activity->activity_type = $testiId ? 'Updated project' : 'created project';
            $activity->ip_address = $geolocation->ip;
            $activity->browser = $agent->browser();
            $activity->device = $agent->device();
            $activity->save();

            $message = $testiId ? lang('Project updated successfully.', 'alerts') : lang('Project created successfully.', 'alerts');
            return response()->json(['code' => 200, 'success' => $message, 'data' => $project], 200);
        } else {
            return Response::json(['errors' => $validator->errors()]);
        }
    }

    public function show($id)
    {
        $this->authorize('Project Edit');
        $post = Projects::find($id);

        $cat = DB::table("category_category_user")->where("category_category_user.category_id", $id)
            ->pluck('category_category_user.category_user_id', 'category_category_user.category_user_id')
            ->all();
        if (request()->ajax()) {
            $output = '';
            $data = Category::all();
            $total_row = $data->count();
            if ($total_row > 0) {
                foreach ($data as $row) {
                    $output .= '


                    <option  value="' . $row->id . '" >' . $row->name . '</option>

                    ';
                }
            }
        }

        return response()->json($post);
    }

    public function destroy($id)
    {
        $this->authorize('Project Delete');
        $testimonial = Projects::find($id);
        $testimonial->delete();

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted a project';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();

        return response()->json(['success' => lang('The project was successfully deleted.', 'alerts')]);
    }

    public function projectmassdestroy(Request $request)
    {
        $this->authorize('Project Delete');
        $student_id_array = $request->input('id');

        $projects = Projects::whereIn('id', $student_id_array)->get();

        foreach ($projects as $project) {
            $project->delete();
        }

        $geolocation = GeoIP::getLocation(request()->getClientIp());
        $agent = new Agent();
        $activity = new EmployeeActivity();
        $activity->user_id = Auth::user()->id;
        $activity->activity_type = 'Deleted Multiple projects';
        $activity->ip_address = $geolocation->ip;
        $activity->browser = $agent->browser();
        $activity->device = $agent->device();
        $activity->save();
        return response()->json(['success' => lang('The project was successfully deleted.', 'alerts')]);
    }

    public function projectlist()
    {

        $category = Category::all();

        $project = Projects::all();

        return response()->json(['category' => $category, 'project' => $project]);
    }

    public function projectassignee(Request $r)
    {

        $this->authorize('Project Assign');
        $projects = Projects::find($r->projected);
        if (!empty($projects)) {
            foreach ($projects as $project) {
                $project->updated_at = now();
                $project->update();

                if ($r->input('category_id') == null) {
                    $project->projectscategory()->detach($r->input(['category_id']));
                } else {
                    foreach ($r->input('category_id') as $value) {
                        $category_id[] = $value;
                    }
                    $project->projectscategory()->sync($r->input(['category_id']));
                }
            }
            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $agent = new Agent();
            $activity = new EmployeeActivity();
            $activity->user_id = Auth::user()->id;
            $activity->activity_type = 'Assigned projects';
            $activity->ip_address = $geolocation->ip;
            $activity->browser = $agent->browser();
            $activity->device = $agent->device();
            $activity->save();
            return redirect()->back()->with(['success' => lang('The projects were successfully assigned.', 'alerts')]);
            // return response()->json(['success' => true, 'message' => lang('The projects were successfully assigned.', 'alerts')]);
        } else {
            return redirect()->back()->with(['error' => lang('Projects have not been assigned.', 'alerts')]);
            // return response()->json(['success' => false, 'message' => lang('Projects have not been assigned.', 'alerts')]);
        }
    }


    public function projetimport()
    {
        $this->authorize('Project Importlist');

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        return view('admin.projects.projectimport')->with($data);
    }


    /**
     * @return \Illuminate\Support\Collection
     */
    public function projetcsv(Request $req)
    {
        $this->authorize('Project Importlist');
        if ($req->hasFile('file')) {
            $validator = Validator::make(
                $req->all(),
                [

                    'file' => 'required|file|mimes:xlsx,csv,txt',
                ],
                [
                    'file.mimes' => 'The file must be a file of type: xlsx, csv.',

                ]
            );
            if ($validator->fails()) {
                return redirect()->back()->withErrors($validator)->withInput();
            }

            $file = $req->file('file')->store('import');

            $import = new ProjectImport;
            $import->import($file);
            if ($import->hasData) {
                $geolocation = GeoIP::getLocation(request()->getClientIp());
                $agent = new Agent();
                $activity = new EmployeeActivity();
                $activity->user_id = Auth::user()->id;
                $activity->activity_type = 'Imported the projects list';
                $activity->ip_address = $geolocation->ip;
                $activity->browser = $agent->browser();
                $activity->device = $agent->device();
                $activity->save();
                return redirect()->route('projects')->with('success', lang('The project list was imported successfully.', 'alerts'));
            } else {
                return redirect()->back()->with('error', 'The imported file does not contain any data.');
            }
            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $agent = new Agent();
            $activity = new EmployeeActivity();
            $activity->user_id = Auth::user()->id;
            $activity->activity_type = 'Imported the projects list';
            $activity->ip_address = $geolocation->ip;
            $activity->browser = $agent->browser();
            $activity->device = $agent->device();
            $activity->save();
            return redirect()->route('projects')->with('success', lang('The project list was imported successfully.', 'alerts'));
        } else {
            return redirect()->back()->with('error', 'Please select file to import data of Project.');
        }
    }

    public function notificationpage()
    {

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $notifications = auth()->user()->notifications()->paginate('10')->groupBy(function ($date) {
            return \Carbon\Carbon::parse($date->created_at)->format('Y-m-d');
        });
        $data['notifications'] = $notifications;

        // $arraystatus = array('mail');
        // $filter =  auth()->user()->notifications()->whereIn('data->status', $arraystatus)->where(function ($query) {
        //     $query->where('data->title', 'Sed repellendus in eligendi quo.')
        //         ->orWhere('data->mailsubject', 'gjfgfgfgfgfghfgghfg');
        // })->get();
        // $filter1 =  auth()->user()->notifications()->where(function ($query) {
        //     $keyword = request()->get('data');
        //     $query->where('data->title', 'LIKE', "%{$keyword}%")
        //         ->orWhere('data->mailsubject', 'LIKE', "%{$keyword}%")
        //         ->orWhere('data->mailtext', 'LIKE', "%{$keyword}%");
        // })->get();

        // $filter2 =  auth()->user()->notifications()->whereIn('data->status', $arraystatus)->get();

        return view('admin.notificationpage')->with($data);
    }
}
