<?php

namespace App\Http\Controllers;

use helpers;
use App\Models\Apptitle;
use App\Models\Currency;
use App\Models\Customer;
use App\Models\Footertext;
use App\Models\Seosetting;
use Jenssegers\Agent\Agent;
use Illuminate\Http\Request;
use Torann\GeoIP\Facades\GeoIP;
use App\Models\EmployeeActivity;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class CurrencyController extends Controller
{
    public function index()
    {
        $this->authorize('Currencies Access');

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $currencies = Currency::get();
        $data['currencies'] = $currencies;

        return view('admin.currency.index')->with($data);
    }
   
    public function edit($id)
    {
        try {
            $this->authorize('Currencies Edit');
        $support = Currency::find($id);
        return response()->json(['data' => $support]);
        } catch (\Throwable $th) {

            return response()->json(['exception' => $th->getMessage()]);
        }
    }

    public function delete($id)
    {
        try {
            $this->authorize('Currencies Delete');

        $support = Currency::findOrFail($id);
        $support->delete();
        $geolocation = GeoIP::getLocation(request()->getClientIp());
                $agent = new Agent();
                $activity = new EmployeeActivity();
                $activity->user_id = Auth::user()->id;
                $activity->activity_type = 'Deleted a currency';
                $activity->ip_address = $geolocation->ip;
                $activity->browser = $agent->browser();
                $activity->device = $agent->device();
                $activity->save();

        return response()->json(['success' => lang('The currency was successfully deleted.', 'alerts')]);
        } catch (\Throwable $th) {
            return response()->json(['exception' => $th->getMessage()]);
        }
    }

    public function status(Request $request, $id)
    {
        
        try {
            $this->authorize('Currencies Edit');
        $supID = Currency::find($id);
        $supID->status = $request->status;
        $supID->save();

        $geolocation = GeoIP::getLocation(request()->getClientIp());
                $agent = new Agent();
                $activity = new EmployeeActivity();
                $activity->user_id = Auth::user()->id;
                $activity->activity_type = 'Changed currencies status';
                $activity->ip_address = $geolocation->ip;
                $activity->browser = $agent->browser();
                $activity->device = $agent->device();
                $activity->save();
        return response()->json(['code' => 200, 'success' => lang('Updated successfully', 'alerts')], 200);
        } catch (\Throwable $th) {
            return response()->json(['exception' => $th->getMessage()]);
        }
    }

    public function currencyMassdestroy(Request $request)
    {
        try {
            $this->authorize('Currencies Delete');
        $currency_id_array = $request->input('id');

        $currencies = Currency::whereIn('id', $currency_id_array)->get();

        foreach ($currencies as $currency) {

            $currency->delete();
        }

        $geolocation = GeoIP::getLocation(request()->getClientIp());
                $agent = new Agent();
                $activity = new EmployeeActivity();
                $activity->user_id = Auth::user()->id;
                $activity->activity_type = 'Deleted multiple currencies';
                $activity->ip_address = $geolocation->ip;
                $activity->browser = $agent->browser();
                $activity->device = $agent->device();
                $activity->save();
        return response()->json(['success' => lang('The currency was successfully deleted.', 'alerts')]);
        } catch (\Throwable $th) {
            return response()->json(['exception' => $th->getMessage()]);
        }
    }

    public function changeCurrency(Request $request)
    {
        $currency = Currency::find($request->currencyId);
        if (Auth::guard('customer')->user()) {
            $customer = Customer::find(Auth::guard('customer')->user()->id);
            $customer->update(['currency' => $currency->code]);
        } else {
            session()->put('currency', $currency->code);
        }

        return response()->json(['success' => lang('The currency was changed .', 'alerts')]);
    }

}
