<?php

namespace App\Http\Controllers;

use Auth;
use Mail;
use GeoIP;
use Carbon\Carbon;
use App\Models\User;
use Hashids\Hashids;
use App\Models\Pages;
use App\Jobs\MailSend;
use App\Models\Groups;
use App\Models\APIData;
use App\Models\CCMAILS;
use App\Models\Holiday;
use App\Models\AddSpace;
use App\Models\Apptitle;
use App\Models\Customer;
use App\Models\Projects;
use App\Models\VerifyOtp;
use App\Models\Footertext;
use App\Models\Seosetting;
use App\Models\Userrating;
use App\Models\Customfield;
use App\Models\Ratingtoken;
use Illuminate\Support\Str;
use App\Models\Announcement;
use Illuminate\Http\Request;
use App\Models\Ticket\Ticket;
use App\Models\tickethistory;
use App\Mail\mailmailablesend;
use App\Models\CategoryEnvato;
use App\Models\Employeerating;
use App\Models\Ticket\Comment;
use App\Models\CustomerSetting;
use App\Models\Ticket\Category;
use App\Models\Articles\Article;
use App\Models\Subcategorychild;
use App\Events\ConversationEvent;
use App\Models\SocialAuthSetting;
use App\Models\TicketCustomfield;
use Illuminate\Support\Facades\Log;
use App\Notifications\TicketCreateNotifications;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

class GuestticketController extends Controller
{
    public function index()
    {
        if (setting('GUEST_TICKET') == 'yes') {
            $tickets = Ticket::paginate(10);
            $categories = Category::whereIn('display', ['ticket', 'all'])->where('status', '1')
                ->get();
            $data['categories'] = $categories;

            $title = Apptitle::first();
            $data['title'] = $title;

            $seopage = Seosetting::first();
            $data['seopage'] = $seopage;

            $post = Pages::all();
            $data['page'] = $post;

            $footertext = Footertext::first();
            $data['footertext'] = $footertext;

            $socialAuthSettings = SocialAuthSetting::first();
            $data['socialAuthSettings'] = $socialAuthSettings;

            $recentarticles = Article::latest()->paginate(5);
            $data['recentarticles'] = $recentarticles;

            $populararticle = Article::orderBy('views', 'desc')->latest()->paginate(5);
            $data['populararticles'] = $populararticle;

            $customfields = Customfield::whereIn('displaytypes', ['both', 'createticket'])->where('status', '1')->get();
            $data['customfields'] = $customfields;

            $now = now();
            $announcement = announcement::whereDate('enddate', '>=', $now->toDateString())->whereDate('startdate', '<=', $now->toDateString())->get();
            $data['announcement'] = $announcement;

            $announcements = Announcement::whereNotNull('announcementday')->get();
            $data['announcements'] = $announcements;

            // $holidays = Holiday::whereDate('startdate', '<=', $now->toDateString())->whereDate('enddate', '>=', $now->toDateString())->where('status',1)->get();
            // $data['holidays'] =  $holidays;

            $currentYear = $now->year;
            $activeHolidays = Holiday::where('status', 1)->get();

            $addspace = AddSpace::where('status', 1)->get();

            // Filter holidays
            $data['holidays'] = $activeHolidays->filter(function ($holiday) use ($now, $currentYear) {
                $holidayStart = Carbon::parse($holiday->startdate);
                $holidayEnd = Carbon::parse($holiday->enddate);

                // Check for fixed holidays (compare only day and month)
                if ($holiday->is_fixed) {
                    return $holidayStart->month === $now->month && $holidayStart->day === $now->day;
                }

                // For non-fixed holidays, check the full date
                return $holidayStart->isSameDay($now);
            });

            if (setting('GUEST_TICKET_OTP') == 'no') {

                return view('guestticket.index', compact('tickets', 'categories'))->with($data);
            } else {

                return view('guestticket.guestticketnootp', compact('tickets', 'categories'))->with($data);
            }
        } else {
            return redirect()->back()->with('error', 'You cannot have access for this guest ticket create.');
        }
    }

    // Guest Ticket Creating
    public function gueststore(Request $request)
    {

        // guest restrict to create tickets based on allowed to create.
        if (setting('RESTRICT_TO_CREATE_TICKET') == 'on' && setting('MAXIMUM_ALLOW_TICKETS') > 0) {
            $customers = Customer::where('email', $request->email)->first();
            if ($customers != null && $customers->tickets != null) {
                $star1 = now()->subHour(setting('MAXIMUM_ALLOW_HOURS'));
                $star2 = now();
                $latestReplies = $customers->tickets()->whereBetween('created_at', [$star1, $star2])->get()->take(setting('MAXIMUM_ALLOW_TICKETS'));
                if ($latestReplies->isNotEmpty()) {
                    $totalcount = 0;
                    foreach ($latestReplies as $comment) {
                        if ($comment->user_id !== null) {
                            $totalcount++;
                        }
                    }
                    if (!$totalcount) {
                        $currentTcikets = $latestReplies->last();
                        $data['difference'] = $currentTcikets->created_at->addHour(setting('REPLY_ALLOW_IN_HOURS'))->diffForHumans(now());

                        $ticketscount = Ticket::where('cust_id', $customers->id)->whereBetween('created_at', [$star1, $star2])->count();
                        if ($ticketscount >= setting('MAXIMUM_ALLOW_TICKETS')) {
                            return response()->json(['error' => lang('Customer has reached maximum allowed tickets to create.', 'alerts'), 'ticcreaterestrict' => 'on']);
                        }
                    }
                }
            }
        }

        $categories = CategoryEnvato::where('category_id', $request->category)->first();

        if (setting('ENVATO_ON') == 'on' && $categories != null) {
            if ($request->envato_id == 'undefined' || $request->envato_id == null || isset($request->envato_id) == false) {
                return response()->json(['message' => 'envatoerror', 'error' => lang('Please enter valid details to create a ticket.', 'alerts')], 200);
            }
        }

        $subcategoriess = Subcategorychild::where('category_id', $request->category)->pluck('subcategory_id')->toArray();
        if ($subcategoriess != null && $request->subscategory != null && !in_array($request->subscategory, $subcategoriess)) {
            return response()->json(['message' => 'subcaterror', 'error' => lang('Please enter valid details to create a ticket.', 'alerts')], 200);
        }

        if ($request->session()->has('sessionemail')) {
            $emailvalidate = $request->session()->get('sessionemail');
        } else {
            $emailvalidate = null;
        }

        // $message = trim(str_replace('&nbsp;', ' ', $request->input('message')));
        // $request->merge(['message' => $message]);

        if (setting('CAPTCHATYPE') == 'off') {
            $this->validate($request, [
                'subject' => 'required|max:255',
                'category' => 'required',
                'message' => 'required|no_script_tags|string',
                'verifyotp' => 'required',
                'agree_terms' =>  'required|in:agreed',

            ]);
        } else {
            if (setting('CAPTCHATYPE') == 'manual') {
                if (setting('RECAPTCH_ENABLE_GUEST') == 'yes') {
                    $request->validate([
                        'subject' => 'required|max:255',
                        'category' => 'required',
                        'message' => 'required|no_script_tags|string',
                        'captcha' => ['required', 'captcha'],
                        // 'captcha' => 'required',
                        'verifyotp' => 'required',
                        'agree_terms' =>  'required|in:agreed',
                    ]);
                } else {
                    $request->validate([
                        'subject' => 'required|max:255',
                        'category' => 'required',
                        'message' => 'required',
                        'verifyotp' => 'required',
                        'agree_terms' =>  'required|in:agreed',

                    ]);
                }
            }
            if (setting('CAPTCHATYPE') == 'google') {
                if (setting('RECAPTCH_ENABLE_GUEST') == 'yes') {
                    $request->validate([
                        'subject' => 'required|max:255',
                        'category' => 'required',
                        'message' => 'required',
                        'g-recaptcha-response' => 'required|recaptcha',
                        'verifyotp' => 'required',
                        'agree_terms' =>  'required|in:agreed',

                    ]);
                } else {
                    $request->validate([
                        'subject' => 'required|max:255',
                        'category' => 'required',
                        'message' => 'required',
                        'verifyotp' => 'required',
                        'agree_terms' =>  'required|in:agreed',

                    ]);
                }
            }
        }

        $guest = Customer::where('email', $emailvalidate)->first();
        if ($guest) {
            if ($guest->userType == 'Guest') {
                $ticket = Ticket::create([
                    'subject' => $request->input('subject'),
                    'cust_id' => $guest->id,
                    'category_id' => $request->input('category'),
                    'message' => $request->input('message'),
                    'project' => $request->input('project'),
                    'status' => 'New',
                ]);
                $ticket = Ticket::find($ticket->id);
                $ticket->ticket_id = setting('CUSTOMER_TICKETID') . 'G-' . $ticket->id;

                $categories = CategoryEnvato::where('category_id', $request->category)->first();
                if ($request->input('envato_id') &&  $categories) {

                    $ticket->purchasecode = encrypt($request->input('envato_id'));
                    if ($request->input('productname')) {
                        $ticket->item_name = $request->input('productname');
                    }
                }
                if ($request->input('envato_support')) {

                    $ticket->purchasecodesupport = $request->input('envato_support');
                }
                $categoryfind = Category::find($request->category);
                $ticket->priority = $categoryfind->priority;
                if ($request->subscategory) {
                    $ticket->subcategory = $request->subscategory;
                }

                // Auto Overdue Ticket
                if (setting('AUTO_OVERDUE_TICKET') == 'no') {
                    $ticket->auto_overdue_ticket = null;
                    $ticket->overduestatus = null;
                } else {
                    if (setting('AUTO_OVERDUE_TICKET_TIME') == '0') {
                        $ticket->auto_overdue_ticket = null;
                        $ticket->overduestatus = null;
                    } else {

                        if ($ticket->status == 'Closed') {
                            $ticket->auto_overdue_ticket = null;
                            $ticket->overduestatus = null;
                        } else {
                            $ticket->auto_overdue_ticket = now()->addDays(setting('AUTO_OVERDUE_TICKET_TIME'));
                            $ticket->overduestatus = null;
                        }
                    }
                }
                // End Auto Overdue Ticket
                $ticket->update();

                $geolocation = GeoIP::getLocation(request()->getClientIp());
                $custupdate = Customer::find($ticket->cust_id);
                $custupdate->last_login_ip = $geolocation->ip;
                $custupdate->timezone = $geolocation->timezone;
                $custupdate->country = $geolocation->country;
                $custupdate->update();

                $customfields = Customfield::whereIn('displaytypes', ['both', 'createticket'])->where('status', 1)->get();

                foreach ($customfields as $customfield) {
                    $ticketcustomfield = new TicketCustomfield();
                    $ticketcustomfield->ticket_id = $ticket->id;
                    $ticketcustomfield->fieldnames = $customfield->fieldnames;
                    $ticketcustomfield->fieldtypes = $customfield->fieldtypes;
                    $ticketcustomfield->fieldoptions = $customfield->fieldoptions;
                    if ($customfield->fieldtypes == 'checkbox') {
                        if ($request->input('custom_' . $customfield->id) != null) {

                            $string = implode(',', $request->input('custom_' . $customfield->id));
                            $ticketcustomfield->values = $string;
                        }
                    }
                    if ($customfield->fieldtypes != 'checkbox') {
                        if ($customfield->fieldprivacy == '1') {
                            $ticketcustomfield->privacymode = $customfield->fieldprivacy;
                            $ticketcustomfield->values = encrypt($request->input('custom_' . $customfield->id));
                        } else {

                            $ticketcustomfield->values = $request->input('custom_' . $customfield->id);
                        }
                    }
                    $ticketcustomfield->save();
                }

                $ccmails = new CCMAILS();
                $ccmails->ticket_id = $ticket->id;
                $ccmails->ccemails = $request->ccmail;
                $ccmails->save();

                foreach ($request->input('ticket', []) as $file) {
                    $provider =  storage()->provider;
                    $provider::mediaupload($ticket, 'uploads/guestticket/' . $file, 'ticket');
                }

                $tickethistory = new tickethistory();
                $tickethistory->ticket_id = $ticket->id;

                $tickethistory->status = $ticket->status;
                $tickethistory->currentAction = 'Ticket Created';
                if ($request->input('ticket_source') != null && $request->input('ticket_source') == 'Livechat') {
                    $tickethistory->username = Auth::user()->name;
                    $tickethistory->type = Auth::user()->getRoleNames()[0];
                } else {
                    $tickethistory->username = $ticket->cust->username;
                    $tickethistory->type = $ticket->cust->userType;
                }

                $tickethistory->save();

                // Create a New ticket reply
                if ($ticket->category) {
                    $notificationcat = $ticket->category->groupscategoryc()->get();
                    $groupIds = $notificationcat->pluck('group_id')->toArray();
                    $groupstatus = false;
                    foreach ($groupIds as $groupid) {
                        $groupexist = Groups::where('groupstatus', '1')->find($groupid);
                        if ($groupexist) {
                            $groupstatus = true;
                        }
                    }

                    $icc = array();

                    if ($groupstatus) {

                        foreach ($notificationcat as $igc) {
                            $groups = $igc->groupsc()
                                ->where('groupstatus', 1)
                                ->with('groupsuser')
                                ->get();

                            foreach ($groups as $group) {
                                $users = $group->groupsuser;

                                foreach ($users as $user) {
                                    $icc[] = $user->users_id;
                                }
                            }
                        }

                        if (!$icc) {
                            $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->get();
                            foreach ($admins as $admin) {
                                $admin->notify(new TicketCreateNotifications($ticket));
                            }
                        } else {

                            $user = User::whereIn('id', $icc)->get();
                            foreach ($user as $users) {
                                $users->notify(new TicketCreateNotifications($ticket));
                            }
                            $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->get();
                            foreach ($admins as $admin) {
                                $admin->notify(new TicketCreateNotifications($ticket));
                            }
                        }
                    } else {
                        foreach (usersdata() as $admin) {
                            $admin->notify(new TicketCreateNotifications($ticket));
                        }
                    }
                }
                $verifyUser = VerifyOtp::where('type', 'guestotp')->where('otp', $request->verifyotp)->first();
                if ($verifyUser) {

                    $verifyUser->delete();
                }

                $guestdetailssession = session()->put('guestdetailssession', $ticket->id);
                $guestimageaccess = session()->put('guestimageaccess', $ticket->id);

                $ccemailsend = CCMAILS::where('ticket_id', $ticket->id)->first();

                $hashids = app(Hashids::class);
                $ticket->encrypted_id = $hashids->encode($ticket->id);

                $ticketData = [
                    'ticket_username' => $ticket->cust->username,
                    'ticket_id' => $ticket->ticket_id,
                    'ticket_title' => $ticket->subject,
                    'ticket_status' => $ticket->status,
                    'ticket_description' => $ticket->message,
                    'ticket_customer_url' => route('guest.ticketdetailshow', $ticket->encrypted_id),
                    'ticket_admin_url' => url('/admin/ticket-view/' . $ticket->encrypted_id),
                    'ticket_category' => $ticket->category ? $ticket->category->name : null,
                    'status' => $ticket->status,
                    'comment' => $request->input('message')
                ];
                try {

                    // $today = Carbon::today();
                    // $holidays = Holiday::whereDate('startdate', '<=', $today)->whereDate('enddate', '>=', $today)->where('status','1')->get();

                    $today = Carbon::today();
                    $activeHolidays = Holiday::where('status', 1)->get();
                    $now = now();
                    $currentYear = $now->year;
                    $holidays = $activeHolidays->filter(function ($holiday) use ($now, $currentYear) {
                        $holidayStart = Carbon::parse($holiday->startdate);
                        $holidayEnd = Carbon::parse($holiday->enddate);
                        if ($holiday->is_fixed) {
                            return $holidayStart->month === $now->month && $holidayStart->day === $now->day;
                        }
                        return $holidayStart->isSameDay($now);
                    });

                    if ($holidays->isNotEmpty() && setting('24hoursbusinessswitch') != 'on') {
                        dispatch((new MailSend($ticket->cust->email, 'customer_send_ticket_created_that_holiday_or_announcement', $ticketData)));
                        if ($ccemailsend->ccemails != null) {
                            dispatch((new MailSend($ccemailsend->ccemails, 'customer_send_ticket_created_that_holiday_or_announcement', $ticketData)));
                        }
                    } else {
                        dispatch((new MailSend($ticket->cust->email, 'customer_send_guestticket_created', $ticketData)));
                        if ($ccemailsend->ccemails != null) {
                            dispatch((new MailSend($ccemailsend->ccemails, 'customer_send_guestticket_created', $ticketData)));
                        }
                    }

                    if ($ticket->category) {
                        $notificationcat = $ticket->category->groupscategoryc()->get();
                        $groupIds = $notificationcat->pluck('group_id')->toArray();
                        $groupstatus = false;
                        foreach ($groupIds as $groupid) {
                            $groupexist = Groups::where('groupstatus', '1')->find($groupid);
                            if ($groupexist) {
                                $groupstatus = true;
                            }
                        }

                        $icc = array();

                        if ($groupstatus) {

                            foreach ($notificationcat as $igc) {
                                $groups = $igc->groupsc()
                                    ->where('groupstatus', 1)
                                    ->with('groupsuser')
                                    ->get();

                                foreach ($groups as $group) {
                                    $users = $group->groupsuser;

                                    foreach ($users as $user) {
                                        $icc[] = $user->users_id;
                                    }
                                }
                            }

                            if (!$icc) {
                                $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                foreach ($admins as $admin) {
                                    if ($admin->usetting->emailnotifyon == 1) {
                                        dispatch((new MailSend($admin->email, 'admin_send_email_ticket_created', $ticketData)));
                                    }
                                }
                            } else {

                                $user = User::whereIn('id', $icc)->where('status', 1)->get();
                                foreach ($user as $users) {
                                    if ($users->usetting->emailnotifyon == 1) {
                                        dispatch((new MailSend($users->email, 'admin_send_email_ticket_created', $ticketData)));
                                    }
                                }
                                $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                foreach ($admins as $admin) {
                                    if ($admin->usetting->emailnotifyon == 1) {
                                        dispatch((new MailSend($admin->email, 'admin_send_email_ticket_created', $ticketData)));
                                    }
                                }
                            }
                        } else {
                            foreach (usersdata() as $admin) {
                                if ($admin->getRoleNames()[0] == 'superadmin' && $admin->usetting->emailnotifyon == 1) {
                                    dispatch((new MailSend($admin->email, 'admin_send_email_ticket_created', $ticketData)));
                                }
                            }
                        }
                    }
                } catch (\Exception $e) {



                    return response()->json(['description' => $ticket->message, 'subject' => $ticket->subject, 'id' => $ticket->id, 'guest' => 'pass', 'data' => $ticket, 'success' => lang('A ticket has been opened with the ticket ID', 'alerts') . $ticket->ticket_id], 200);
                }

                return response()->json(['description' => $ticket->message, 'subject' => $ticket->subject, 'id' => $ticket->id, 'guest' => 'pass', 'data' => $ticket, 'success' => lang('A ticket has been opened with the ticket ID', 'alerts') . $ticket->ticket_id], 200);
            }

            if ($guest->userType == 'Customer') {
                return response()->json(['error' => lang('Email is already registered, Please login to create a Ticket', 'alerts'), 'email' => 'already']);
            }
        }
        if (!$guest) {
            return response()->json(['warning' => lang('Email is not registered, Please register the account', 'alerts'), 'email' => 'already']);
        }
    }

    // Guest Ticket Creating1
    public function gueststore1(Request $request)
    {
        // guest restrict to create tickets based on allowed to create.
        if (setting('RESTRICT_TO_CREATE_TICKET') == 'on' && setting('MAXIMUM_ALLOW_TICKETS') > 0) {
            $customers = Customer::where('email', $request->email)->first();
            if ($customers != null && $customers->tickets != null) {
                $star1 = now()->subHour(setting('MAXIMUM_ALLOW_HOURS'));
                $star2 = now();
                $latestReplies = $customers->tickets()->whereBetween('created_at', [$star1, $star2])->get()->take(setting('MAXIMUM_ALLOW_TICKETS'));
                if ($latestReplies->isNotEmpty()) {
                    $totalcount = 0;
                    foreach ($latestReplies as $comment) {
                        if ($comment->user_id !== null) {
                            $totalcount++;
                        }
                    }
                    if (!$totalcount) {
                        $currentTcikets = $latestReplies->last();
                        $data['difference'] = $currentTcikets->created_at->addHour(setting('REPLY_ALLOW_IN_HOURS'))->diffForHumans(now());

                        $ticketscount = Ticket::where('cust_id', $customers->id)->whereBetween('created_at', [$star1, $star2])->count();
                        if ($ticketscount >= setting('MAXIMUM_ALLOW_TICKETS')) {
                            return response()->json(['error' => lang('Customer has reached maximum allowed tickets to create.', 'alerts'), 'ticcreaterestrict' => 'on']);
                        }
                    }
                }
            }
        }

        $categories = CategoryEnvato::where('category_id', $request->category)->first();

        if (setting('ENVATO_ON') == 'on' && $categories != null) {
            if ($request->envato_id == 'undefined' || $request->envato_id == null || isset($request->envato_id) == false) {
                return response()->json(['message' => 'envatoerror', 'error' => lang('Please enter valid details to create a ticket.', 'alerts')], 200);
            }
        }


        $subcategoriess = Subcategorychild::where('category_id', $request->category)->pluck('subcategory_id')->toArray();
        if ($subcategoriess != null && $request->subscategory != null && !in_array($request->subscategory, $subcategoriess)) {
            return response()->json(['message' => 'subcaterror', 'error' => lang('Please enter valid details to create a ticket.', 'alerts')], 200);
        }

        if (setting('GUEST_TICKET') == 'yes') {
            session()->put('guestdetailssession', $request->session()->get('guestdetailssession'));
            if (setting('CAPTCHATYPE') == 'off') {
                $this->validate($request, [
                    'subject' => 'required|max:255',
                    'category' => 'required',
                    'message' => 'required',
                    'email' => 'required|max:255|indisposable|email',
                    'agree_terms' =>  'required|in:agreed',

                ]);
            } else {
                if ($request->input('ticket_source') != null && $request->input('ticket_source') == 'Livechat') {
                    $this->validate($request, [
                        'subject' => 'required|max:255',
                        'category' => 'required',
                        'message' => 'required',
                        'email' => 'required|max:255|indisposable|email',
                    ]);
                } else {
                    if (setting('CAPTCHATYPE') == 'manual') {
                        if (setting('RECAPTCH_ENABLE_GUEST') == 'yes') {
                            $request->validate([
                                'subject' => 'required|max:255',
                                'category' => 'required',
                                'message' => 'required',
                                'email' => 'required|max:255|indisposable|email',
                                'captcha' => ['required', 'captcha'],
                                // 'captcha' => 'required',
                                'agree_terms' =>  'required|in:agreed',
                            ]);
                        } else {
                            $request->validate([
                                'subject' => 'required|max:255',
                                'category' => 'required',
                                'email' => 'required|max:255|indisposable|email',
                                'message' => 'required',
                                'agree_terms' =>  'required|in:agreed',

                            ]);
                        }
                    }
                    if (setting('CAPTCHATYPE') == 'google') {
                        if (setting('RECAPTCH_ENABLE_GUEST') == 'yes') {
                            $request->validate([
                                'subject' => 'required|max:255',
                                'category' => 'required',
                                'message' => 'required',
                                'email' => 'required|max:255|indisposable|email',
                                'g-recaptcha-response' => 'required|recaptcha',
                                'agree_terms' =>  'required|in:agreed',

                            ]);
                        } else {
                            $request->validate([
                                'subject' => 'required|max:255',
                                'category' => 'required',
                                'email' => 'required|max:255|indisposable|email',
                                'message' => 'required',
                                'agree_terms' =>  'required|in:agreed',

                            ]);
                        }
                    }
                }
            }

            $email = $request->email;
            $completeDomain = substr(strrchr($email, "@"), 1);
            $emaildomainlist = setting('EMAILDOMAIN_LIST');
            $emaildomainlistArray = explode(",", $emaildomainlist);

            if (setting('EMAILDOMAIN_BLOCKTYPE') == 'blockemail') {

                if (setting('EMAILDOMAIN_LIST') == null) {
                    $tickets = $this->validateemaildomain($request);
                    if ($tickets == 'customer') {
                        return response()->json(['error' => lang('Email is already registered, Please ask customer to login and create a Ticket', 'alerts'), 'email' => 'already']);
                    }
                    if ($tickets == 'notguest') {
                        return response()->json(['warning' => lang('Email is not registered, Please register the account', 'alerts'), 'email' => 'already']);
                    }

                    session()->put('guestdetailssession', $tickets->id);
                    session()->put('guestimageaccess', $tickets->id);

                    return response()->json(['description' => $tickets->message, 'subject' => $tickets->subject, 'id' => $tickets->id, 'guest' => 'pass', 'data' => $tickets, 'success' => lang('A ticket has been opened with the ticket ID', 'alerts') . $tickets->ticket_id], 200);
                } else {
                    if (in_array($completeDomain, $emaildomainlistArray)) {

                        return response()->json(['message' => 'domainblock', 'error' => lang('Domain is Blocked List', 'alerts')], 200);
                    }
                    $tickets = $this->validateemaildomain($request);
                    if ($tickets == 'customer') {
                        return response()->json(['error' => lang('Email is already registered, Please ask customer to login and create a Ticket', 'alerts'), 'email' => 'already']);
                    }
                    if ($tickets == 'notguest') {
                        return response()->json(['warning' => lang('Email is not registered, Please register the account', 'alerts'), 'email' => 'already']);
                    }

                    session()->put('guestdetailssession', $tickets->id);
                    session()->put('guestimageaccess', $tickets->id);

                    return response()->json(['description' => $tickets->message, 'subject' => $tickets->subject, 'id' => $tickets->id, 'guest' => 'pass', 'data' => $tickets, 'success' => lang('A ticket has been opened with the ticket ID', 'alerts') . $tickets->ticket_id], 200);
                }
            }

            if (setting('EMAILDOMAIN_BLOCKTYPE') == 'allowemail') {
                if (setting('EMAILDOMAIN_LIST') == null) {
                    $tickets = $this->validateemaildomain($request);
                    if ($tickets == 'customer') {
                        return response()->json(['error' => lang('Email is already registered, Please login to create a Ticket', 'alerts'), 'email' => 'already']);
                    }
                    if ($tickets == 'notguest') {
                        return response()->json(['warning' => lang('Email is not registered, Please register the account', 'alerts'), 'email' => 'already']);
                    }
                    $guestdetailssession = session()->put('guestdetailssession', $tickets->id);
                    return response()->json(['guest' => 'pass', 'data' => $tickets, 'success' => lang('A ticket has been opened with the ticket ID', 'alerts') . $tickets->ticket_id], 200);
                } else {
                    if (in_array($completeDomain, $emaildomainlistArray)) {
                        $tickets = $this->validateemaildomain($request);
                        if ($tickets == 'customer') {
                            return response()->json(['error' => lang('Email is already registered, Please login to create a Ticket', 'alerts'), 'email' => 'already']);
                        }
                        if ($tickets == 'notguest') {
                            return response()->json(['warning' => lang('Email is not registered, Please register the account', 'alerts'), 'email' => 'already']);
                        }
                        $guestdetailssession = session()->put('guestdetailssession', $tickets->id);
                        return response()->json(['guest' => 'pass', 'data' => $tickets, 'success' => lang('A ticket has been opened with the ticket ID', 'alerts') . $tickets->ticket_id], 200);
                    }
                    return response()->json(['message' => 'domainblock', 'error' => lang('Domain is Blocked List', 'alerts')], 200);
                }
            }
        } else {
            return response()->json(['message' => 'accessdenied', 'error' => lang('you are not authorised to this ticket create', 'alerts')], 500);
        }
    }

    private function validateemaildomain($request)
    {

        $userexits = Customer::where('email', $request->email)->count();
        if ($userexits == 1) {
            $guest = Customer::where('email', $request->email)->first();
        } else {
            if ($request->input('ticket_source') != null && $request->input('ticket_source') == 'Livechat') {
                $custtype = 'Livechat';
                $custname = $request->input('username');
            } else {
                $custtype = 'Guest';
                $custname = 'GUEST';
            }

            $guest = Customer::create([

                'firstname' => '',
                'lastname' => '',
                'username' => $custname,
                'email' => $request->email,
                'userType' => $custtype,
                'password' => null,
                'status' => '1',
                'image' => null,

            ]);
            $customersetting = new CustomerSetting();
            $customersetting->custs_id = $guest->id;
            $customersetting->save();
        }
        $guest = Customer::where('email', $request->email)->first();
        if (!$guest) {
            return 'notguest';
        }
        if ($guest) {

            if ($guest->userType == 'Customer') {

                return 'customer';
            }

            $ticket = Ticket::create([
                'subject' => $request->input('subject'),
                'cust_id' => $guest->id,
                'category_id' => $request->input('category'),
                'message' => $request->input('message'),
                'project' => $request->input('project'),
                'tickettype' => $request->input('ticket_source') != null ? $request->input('ticket_source') : null,
                'status' => 'New',
            ]);
            $ticket = Ticket::find($ticket->id);
            $ticket->ticket_id = setting('CUSTOMER_TICKETID') . 'G-' . $ticket->id;

            $categories = CategoryEnvato::where('category_id', $request->category)->first();
            if ($request->input('envato_id') &&  $categories) {

                $ticket->purchasecode = encrypt($request->input('envato_id'));
                if ($request->input('productname')) {
                    $ticket->item_name = $request->input('productname');
                }
            }
            if ($request->input('envato_support')) {

                $ticket->purchasecodesupport = $request->input('envato_support');
            }
            $categoryfind = Category::find($request->category);
            $ticket->priority = $categoryfind->priority;
            if ($request->subscategory) {
                $ticket->subcategory = $request->subscategory;
            }

            // Auto Overdue Ticket
            if (setting('AUTO_OVERDUE_TICKET') == 'no') {
                $ticket->auto_overdue_ticket = null;
                $ticket->overduestatus = null;
            } else {
                if (setting('AUTO_OVERDUE_TICKET_TIME') == '0') {
                    $ticket->auto_overdue_ticket = null;
                    $ticket->overduestatus = null;
                } else {

                    if ($ticket->status == 'Closed') {
                        $ticket->auto_overdue_ticket = null;
                        $ticket->overduestatus = null;
                    } else {
                        $ticket->auto_overdue_ticket = now()->addDays(setting('AUTO_OVERDUE_TICKET_TIME'));
                        $ticket->overduestatus = null;
                    }
                }
            }
            // End Auto Overdue Ticket

            $ticket->update();
            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $custupdate = Customer::find($ticket->cust_id);
            $custupdate->last_login_ip = $geolocation->ip;
            $custupdate->timezone = $geolocation->timezone;
            $custupdate->country = $geolocation->country;
            $custupdate->update();

            $customfields = Customfield::whereIn('displaytypes', ['both', 'createticket'])->where('status', 1)->get();

            foreach ($customfields as $customfield) {
                $ticketcustomfield = new TicketCustomfield();
                $ticketcustomfield->ticket_id = $ticket->id;
                $ticketcustomfield->fieldnames = $customfield->fieldnames;
                $ticketcustomfield->fieldtypes = $customfield->fieldtypes;
                $ticketcustomfield->fieldoptions = $customfield->fieldoptions;
                if ($customfield->fieldtypes == 'checkbox') {
                    if ($request->input('custom_' . $customfield->id) != null) {

                        $string = implode(',', $request->input('custom_' . $customfield->id));
                        $ticketcustomfield->values = $string;
                    }
                }
                if ($customfield->fieldtypes != 'checkbox') {
                    if ($customfield->fieldprivacy == '1') {
                        $ticketcustomfield->privacymode = $customfield->fieldprivacy;
                        $ticketcustomfield->values = encrypt($request->input('custom_' . $customfield->id));
                    } else {

                        $ticketcustomfield->values = $request->input('custom_' . $customfield->id);
                    }
                }
                $ticketcustomfield->save();
            }

            $ccmails = new CCMAILS();
            $ccmails->ticket_id = $ticket->id;
            $ccmails->ccemails = $request->ccmail;
            $ccmails->save();

            foreach ($request->input('ticket', []) as $file) {
                $provider =  storage()->provider;
                if ($request->input('ticket_source') == 'Livechat') {
                    $provider::mediaupload($ticket, 'uploads/livechat/' . $file, 'ticket');
                } else {
                    $provider::mediaupload($ticket, 'uploads/guestticket/' . $file, 'ticket');
                }
            }

            $tickethistory = new tickethistory();
            $tickethistory->ticket_id = $ticket->id;

            $tickethistory->status = $ticket->status;
            $tickethistory->currentAction = 'Ticket Created';
            if ($request->input('ticket_source') != null && $request->input('ticket_source') == 'Livechat') {
                $tickethistory->username = Auth::user()->name;
                $tickethistory->type = Auth::user()->getRoleNames()[0];
            } else {
                $tickethistory->username = $ticket->cust->username;
                $tickethistory->type = $ticket->cust->userType;
            }

            $tickethistory->save();

            // Create a New ticket reply
            if ($ticket->category) {
                $notificationcat = $ticket->category->groupscategoryc()->get();
                $groupIds = $notificationcat->pluck('group_id')->toArray();
                $groupstatus = false;
                foreach ($groupIds as $groupid) {
                    $groupexist = Groups::where('groupstatus', '1')->find($groupid);
                    if ($groupexist) {
                        $groupstatus = true;
                    }
                }

                $icc = array();

                if ($groupstatus) {

                    foreach ($notificationcat as $igc) {
                        $groups = $igc->groupsc()
                            ->where('groupstatus', 1)
                            ->with('groupsuser')
                            ->get();

                        foreach ($groups as $group) {
                            $users = $group->groupsuser;

                            foreach ($users as $user) {
                                $icc[] = $user->users_id;
                            }
                        }
                    }

                    if (!$icc) {
                        $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->get();
                        foreach ($admins as $admin) {
                            $admin->notify(new TicketCreateNotifications($ticket));
                        }
                    } else {

                        $user = User::whereIn('id', $icc)->get();
                        foreach ($user as $users) {
                            $users->notify(new TicketCreateNotifications($ticket));
                        }
                        $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->get();
                        foreach ($admins as $admin) {
                            $admin->notify(new TicketCreateNotifications($ticket));
                        }
                    }
                } else {
                    foreach (usersdata() as $admin) {
                        $admin->notify(new TicketCreateNotifications($ticket));
                    }
                }
            }
            $ccemailsend = CCMAILS::where('ticket_id', $ticket->id)->first();
            $hashids = app(Hashids::class);
            $ticket->encrypted_id = $hashids->encode($ticket->id);
            $ticketData = [
                'ticket_username' => $ticket->cust->username,
                'ticket_title' => $ticket->subject,
                'ticket_id' => $ticket->ticket_id,
                'ticket_status' => $ticket->status,
                'ticket_description' => $ticket->message,
                'ticket_customer_url' => route('guest.ticketdetailshow', $ticket->encrypted_id),
                'ticket_admin_url' => url('/admin/ticket-view/' . $ticket->encrypted_id),
                'ticket_category' => $ticket->category ? $ticket->category->name : null,
                'status' => $ticket->status,
                'comment' => $request->input('message')
            ];

            $ticket['customer_url'] = $ticketData['ticket_customer_url'];

            try {
                // $today = Carbon::today();
                // $holidays = Holiday::whereDate('startdate', '<=', $today)->whereDate('enddate', '>=', $today)->where('status','1')->get();

                $today = Carbon::today();
                $activeHolidays = Holiday::where('status', 1)->get();
                $now = now();
                $currentYear = $now->year;
                $holidays = $activeHolidays->filter(function ($holiday) use ($now, $currentYear) {
                    $holidayStart = Carbon::parse($holiday->startdate);
                    $holidayEnd = Carbon::parse($holiday->enddate);
                    if ($holiday->is_fixed) {
                        return $holidayStart->month === $now->month && $holidayStart->day === $now->day;
                    }
                    return $holidayStart->isSameDay($now);
                });

                if ($holidays->isNotEmpty() && setting('24hoursbusinessswitch') != 'on') {
                    dispatch((new MailSend($ticket->cust->email, 'customer_send_ticket_created_that_holiday_or_announcement', $ticketData)));
                    if ($ccemailsend->ccemails != null) {
                        dispatch((new MailSend($ccemailsend->ccemails, 'customer_send_ticket_created_that_holiday_or_announcement', $ticketData)));
                    }
                } else {
                    dispatch((new MailSend($ticket->cust->email, 'customer_send_guestticket_created', $ticketData)));
                    if ($ccemailsend->ccemails != null) {
                        dispatch((new MailSend($ccemailsend->ccemails, 'customer_send_guestticket_created', $ticketData)));
                    }
                }

                if ($ticket->category) {
                    $notificationcat = $ticket->category->groupscategoryc()->get();
                    $groupIds = $notificationcat->pluck('group_id')->toArray();
                    $groupstatus = false;
                    foreach ($groupIds as $groupid) {
                        $groupexist = Groups::where('groupstatus', '1')->find($groupid);
                        if ($groupexist) {
                            $groupstatus = true;
                        }
                    }

                    $icc = array();

                    if ($groupstatus) {

                        foreach ($notificationcat as $igc) {
                            $groups = $igc->groupsc()
                                ->where('groupstatus', 1)
                                ->with('groupsuser')
                                ->get();

                            foreach ($groups as $group) {
                                $users = $group->groupsuser;

                                foreach ($users as $user) {
                                    $icc[] = $user->users_id;
                                }
                            }
                        }
                        if (!$icc) {
                            $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                            foreach ($admins as $admin) {
                                if ($admin->usetting->emailnotifyon == 1) {
                                    dispatch((new MailSend($admin->email, 'admin_send_email_ticket_created', $ticketData)));
                                }
                            }
                        } else {

                            $user = User::whereIn('id', $icc)->where('status', 1)->get();
                            foreach ($user as $users) {
                                if ($users->usetting->emailnotifyon == 1) {
                                    dispatch((new MailSend($users->email, 'admin_send_email_ticket_created', $ticketData)));
                                }
                            }
                            $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                            foreach ($admins as $admin) {
                                if ($admin->getRoleNames()[0] == 'superadmin' && $admin->usetting->emailnotifyon == 1) {
                                    dispatch((new MailSend($admin->email, 'admin_send_email_ticket_created', $ticketData)));
                                }
                            }
                        }
                    } else {
                        foreach (usersdata() as $admin) {
                            if ($admin->usetting->emailnotifyon == 1) {
                                dispatch((new MailSend($admin->email, 'admin_send_email_ticket_created', $ticketData)));
                            }
                        }
                    }
                }
            } catch (\Exception $e) {

                return $ticket;
            }

            return $ticket;
        }
    }

    public function guestdetails($id)
    {

        $ticket = Ticket::find($id);
        $data['ticket'] = $ticket;

        $title = Apptitle::first();
        $data['title'] = $title;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $socialAuthSettings = SocialAuthSetting::first();
        $data['socialAuthSettings'] = $socialAuthSettings;

        return view('guestticket.viewticketdetails')->with($data);
    }

    public function guestmedia(Request $request)
    {
        $path = public_path('uploads/guestticket/');

        if (!file_exists($path)) {
            mkdir($path, 0777, true);
        }

        $file = $request->file('file');

        $name = uniqid() . '_' . trim($file->getClientOriginalName());

        $file->move($path, $name);

        return response()->json([
            'name' => $name,
            'original_name' => $file->getClientOriginalName(),
        ]);
    }

    public function guestview($ticket_id)
    {

        $aaa = Str::length($ticket_id);
        $hashids = app(Hashids::class);
        if ($aaa == 20) {

            $ticket_id = $hashids->decode($ticket_id)[0];

            $ticket = Ticket::where('id', $ticket_id)->firstOrFail();
        } elseif ($aaa > 20) {
            $ticket_id = decrypt($ticket_id);
            $ticket = Ticket::where('ticket_id', $ticket_id)->firstOrFail();
        } elseif ($aaa < 20) {
            $ticket = Ticket::where('ticket_id', $ticket_id)->firstOrFail();
        }

        $ticket->encrypted_ticketId = encrypt($ticket->ticket_id);
        $comments = $ticket->comments()->paginate(5);
        $category = $ticket->category;

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $socialAuthSettings = SocialAuthSetting::first();
        $data['socialAuthSettings'] = $socialAuthSettings;

        $now = now();
        $announcement = announcement::whereDate('enddate', '>=', $now->toDateString())->whereDate('startdate', '<=', $now->toDateString())->get();
        $data['announcement'] = $announcement;

        $announcements = Announcement::whereNotNull('announcementday')->get();
        $data['announcements'] = $announcements;

        // $holidays = Holiday::whereDate('startdate', '<=', $now->toDateString())->whereDate('enddate', '>=', $now->toDateString())->where('status', '1')->get();
        // $data['holidays'] =  $holidays;

        $currentYear = $now->year;
        $activeHolidays = Holiday::where('status', 1)->get();

        // Filter holidays
        $data['holidays'] = $activeHolidays->filter(function ($holiday) use ($now, $currentYear) {
            $holidayStart = Carbon::parse($holiday->startdate);
            $holidayEnd = Carbon::parse($holiday->enddate);

            // Check for fixed holidays (compare only day and month)
            if ($holiday->is_fixed) {
                return $holidayStart->month === $now->month && $holidayStart->day === $now->day;
            }

            // For non-fixed holidays, check the full date
            return $holidayStart->isSameDay($now);
        });

        $data['addspace'] = AddSpace::where('status', 1)->get();

        // customer restrict to reply for the ticket.
        $commentsNull = $ticket->comments()->get();
        if ($commentsNull->all() != null) {

            foreach ($commentsNull as $latestone) {
                if ($latestone->lastseen == null && $latestone->user_id != null) {
                    $latestone->lastseen = now();
                    $latestone->save();
                }
            }
        }

        if (setting('RESTRICT_TO_REPLY_TICKET') == 'on' && $commentsNull->all() != null && setting('MAXIMUM_ALLOW_REPLIES') > 0) {

            $star1 = now()->subHour(setting('REPLY_ALLOW_IN_HOURS'));
            $star2 = now();
            $latestReplies = $ticket->comments()->whereBetween('created_at', [$star1, $star2])->get()->take(setting('MAXIMUM_ALLOW_REPLIES'));
            $totalcount = 0;
            foreach ($latestReplies as $comment) {
                if ($comment->user_id !== null) {
                    $totalcount++;
                }
            }

            if (!$totalcount) {
                $currentTcikets = $latestReplies->last();
                $data['difference'] = $currentTcikets->created_at->addHour(setting('REPLY_ALLOW_IN_HOURS'))->diffForHumans(now());
                $createdcount = $ticket->comments()->where('cust_id', $ticket->cust_id)->whereBetween('created_at', [$star1, $star2])->count();

                if (session()->get('guestdetailssession') == $ticket_id) {
                    if (request()->page) {
                        $view = view('guestticket.showdataticket', compact('comments', 'createdcount', 'ticket'))->render();
                        return response()->json(['html' => $view]);
                    }
                    return view('guestticket.show', compact('ticket', 'category', 'comments', 'createdcount'))->with($data);
                } else {
                    return view('guestticket.shownosession')->with($data);
                }
            }
        }

        $createdcount = '';
        if (session()->get('guestdetailssession') == $ticket_id) {
            if (request()->page) {
                $view = view('guestticket.showdataticket', compact('comments', 'createdcount', 'ticket'))->render();
                return response()->json(['html' => $view]);
            }
            return view('guestticket.show', compact('ticket', 'category', 'comments', 'createdcount'))->with($data);
        } else {
            return view('guestticket.shownosession')->with($data);
        }
    }

    public function postComment(Request $request, $ticket_id)
    {

        $ticket_id = decrypt($ticket_id);
        $ticket = Ticket::where('ticket_id', $ticket_id)->firstOrFail();
        $ticket->encrypted_ticketId = encrypt($ticket->ticket_id);
        if ($ticket->status == "Closed") {

            return redirect()->back()->with("error", lang('The ticket has been already closed.', 'alerts'));
        } else {
            // $comment = trim(str_replace('&nbsp;', ' ', $request->input('comment')));
            // $request->merge(['comment' => $comment]);
            $this->validate($request, [
                'comment' => 'required',
            ]);
            $comment = Comment::create([
                'ticket_id' => $request->input('ticket_id'),
                'cust_id' => $ticket->cust->id,
                'user_id' => null,
                'comment' => $request->input('comment'),
                'commentfrom' => 'guestview',
            ]);
            $geolocation = GeoIP::getLocation(request()->getClientIp());
            $custupdate = Customer::find($ticket->cust_id);
            $custupdate->last_login_ip = $geolocation->ip;
            $custupdate->timezone = $geolocation->timezone;
            $custupdate->country = $geolocation->country;
            $custupdate->update();

            foreach ($request->input('comments', []) as $file) {

                $provider =  storage()->provider;
                $provider::mediaupload($comment, 'uploads/guestticket/' . $file, 'comments');
            }

            // Closing the ticket

            if (request()->has(['status'])) {

                $ticket = Ticket::where('ticket_id', $ticket_id)->firstOrFail();

                $ticket->status = $request->input('status');
                $ticket->closing_ticket = now();
                $ticket->replystatus = 'Replied';
                if ($ticket->fileupload_allowed == 'once') {
                    $ticket->fileupload_allowed = null;
                }
                $ticket->update();

                $ticketOwner = $ticket->user;
            }
            if (webSocketCheck()) {
                try {
                    event(new ConversationEvent(dbname(), $request->comment, $request->status, $ticket->cust, $comment->getMedia('comments'), $comment, 'customer', $ticket_id, $ticket, null));
                } catch (\Exception $e) {
                }
            }

            $ticket = Ticket::where('ticket_id', $ticket_id)->firstOrFail();
            $ticket->last_reply = now();
            if ($ticket->fileupload_allowed == 'once') {
                $ticket->fileupload_allowed = null;
            }
            // Auto Overdue Ticket

            if (setting('AUTO_OVERDUE_TICKET') == 'no') {

                $ticket->auto_overdue_ticket = null;
            } else {
                if (setting('AUTO_OVERDUE_TICKET_TIME') == '0') {

                    $ticket->auto_overdue_ticket = null;
                } else {

                    if ($ticket->status == 'Closed') {
                        $ticket->auto_overdue_ticket = null;
                    } else {
                        $ticket->auto_overdue_ticket = now()->addDays(setting('AUTO_OVERDUE_TICKET_TIME'));
                    }
                }
            }
            // Auto Overdue Ticket

            // Auto Closing Ticket

            if (setting('AUTO_CLOSE_TICKET') == 'no') {

                $ticket->auto_close_ticket = null;
            } else {
                if (setting('AUTO_CLOSE_TICKET_TIME') == '0') {

                    $ticket->auto_close_ticket = null;
                } else {

                    $ticket->auto_close_ticket = null;
                }
            }
            // End Auto Close Ticket

            // Auto Response Ticket

            if (setting('AUTO_RESPONSETIME_TICKET') == 'no') {
                $ticket->auto_replystatus = null;
            } else {
                if (setting('AUTO_RESPONSETIME_TICKET_TIME') == '0') {
                    $ticket->auto_replystatus = null;
                } else {
                    $ticket->auto_replystatus = null;
                }
            }
            // End Auto Response Ticket

            if (request()->input(['status']) == 'Closed') {
                $ticket->replystatus = 'Solved';
            }
            $ticket->update();

            $ccemailsend = CCMAILS::where('ticket_id', $ticket->id)->first();

            if (request()->input(['status']) == 'Closed') {
                $tickethistory = new tickethistory();
                $tickethistory->ticket_id = $ticket->id;

                $tickethistory->ticketnote = $ticket->ticketnote->isNotEmpty();
                $tickethistory->overduestatus = $ticket->overduestatus;
                $tickethistory->status = $ticket->status;
                $tickethistory->replystatus = $ticket->replystatus;
                $tickethistory->currentAction = 'Created';
                $tickethistory->username = $comment->cust->username;
                $tickethistory->type = $comment->cust->userType;

                $tickethistory->save();

                $hashids = app(Hashids::class);
                $ticket->encrypted_id = $hashids->encode($ticket->id);

                $ticketData = [
                    'ticket_username' => $ticket->cust->username,
                    'ticket_id' => $ticket->ticket_id,
                    'ticket_title' => $ticket->subject,
                    'ticket_status' => $ticket->status,
                    'comment' => $comment->comment,
                    'ticket_description' => $ticket->message,
                    'ticket_customer_url' => route('loadmore.load_data', $ticket->encrypted_id),
                    'ticket_admin_url' => url('/admin/ticket-view/' . $ticket->encrypted_id),
                    'ticket_category' => $ticket->category ? $ticket->category->name : null,
                    'status' => $ticket->status
                ];

                try {

                    /**** Close Ticket mail and notificaton ****/
                    if ($ticket->category) {
                        $notificationcat = $ticket->category->groupscategoryc()->get();
                        $groupIds = $notificationcat->pluck('group_id')->toArray();
                        $groupstatus = false;
                        foreach ($groupIds as $groupid) {
                            $groupexist = Groups::where('groupstatus', '1')->find($groupid);
                            if ($groupexist) {
                                $groupstatus = true;
                            }
                        }

                        $icc = array();

                        if ($groupstatus) {

                            foreach ($notificationcat as $igc) {
                                $groups = $igc->groupsc()
                                    ->where('groupstatus', 1)
                                    ->with('groupsuser')
                                    ->get();

                                foreach ($groups as $group) {
                                    $users = $group->groupsuser;

                                    foreach ($users as $user) {
                                        $icc[] = $user->users_id;
                                    }
                                }
                            }


                            if (!$icc) {
                                $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                foreach ($admins as $admin) {
                                    $admin->notify(new TicketCreateNotifications($ticket));
                                    if ($admin->usetting->emailnotifyon == 1) {
                                        dispatch((new MailSend($admin->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                                    }
                                }
                            } else {
                                if ($ticket->myassignuser) {
                                    $assignee = $ticket->ticketassignmutliples;
                                    foreach ($assignee as $assignees) {
                                        $user = User::where('id', $assignees->toassignuser_id)->get();
                                        foreach ($user as $users) {
                                            if ($users->id == $assignees->toassignuser_id && $users->status == '1') {
                                                $users->notify(new TicketCreateNotifications($ticket));
                                                if ($users->usetting->emailnotifyon == 1) {
                                                    dispatch((new MailSend($users->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                                                }
                                            }
                                        }
                                    }
                                } else if ($ticket->selfassignuser_id) {
                                    $self = User::findOrFail($ticket->selfassignuser_id);
                                    $self->notify(new TicketCreateNotifications($ticket));
                                    if ($self->usetting->emailnotifyon == 1) {
                                        dispatch((new MailSend($self->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                                    }
                                } else if ($icc) {
                                    $user = User::whereIn('id', $icc)->where('status', 1)->get();
                                    foreach ($user as $users) {
                                        $users->notify(new TicketCreateNotifications($ticket));
                                        if ($users->usetting->emailnotifyon == 1) {
                                            dispatch((new MailSend($users->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                                        }
                                    }
                                    $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('status', 1)->get();
                                    foreach ($admins as $admin) {
                                        if ($admin->getRoleNames()[0] == 'superadmin') {
                                            $admin->notify(new TicketCreateNotifications($ticket));
                                            if ($admin->usetting->emailnotifyon == 1) {
                                                dispatch((new MailSend($admin->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                                            }
                                        }
                                    }
                                } else {
                                    $users = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                    foreach ($users as $user) {
                                        $user->notify(new TicketCreateNotifications($ticket));
                                        if ($user->usetting->emailnotifyon == 1) {
                                            dispatch((new MailSend($user->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                                        }
                                    }
                                }
                            }
                        } else {
                            if ($ticket->myassignuser) {
                                $assignee = $ticket->ticketassignmutliples;
                                foreach ($assignee as $assignees) {
                                    $user = User::where('id', $assignees->toassignuser_id)->get();
                                    foreach ($user as $users) {
                                        if ($users->id == $assignees->toassignuser_id  && $users->status == '1') {
                                            $users->notify(new TicketCreateNotifications($ticket));
                                            if ($users->usetting->emailnotifyon == 1) {
                                                dispatch((new MailSend($users->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                                            }
                                        }
                                    }
                                }
                            } else if ($ticket->selfassignuser_id) {
                                $self = User::findOrFail($ticket->selfassignuser_id);
                                $self->notify(new TicketCreateNotifications($ticket));
                                if ($self->usetting->emailnotifyon == 1) {
                                    dispatch((new MailSend($self->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                                }
                            } else {
                                foreach (usersdata() as $user) {
                                    $user->notify(new TicketCreateNotifications($ticket));
                                    if ($user->usetting->emailnotifyon == 1) {
                                        dispatch((new MailSend($user->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                                    }
                                }
                            }
                        }
                    } else {

                        $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')
                            ->leftJoin('groups', 'groups.id', 'groups_users.groups_id')
                            ->select('users.id', 'users.firstname', 'users.lastname', 'users.email', 'groups.groupname', 'groups.groupstatus')
                            ->where('users.status', 1)
                            ->where(function ($query) {
                                $query->whereNull('groups_users.groups_id')
                                    ->orWhere('groups.groupstatus', '!=', 1);
                            })
                            ->get();
                        foreach ($admins as $admin) {
                            $admin->notify(new TicketCreateNotifications($ticket));
                            if ($admin->usetting->emailnotifyon == 1) {
                                dispatch((new MailSend($admin->email, 'admin_sendemail_whenticketclosed', $ticketData)));
                            }
                        }
                    }

                    dispatch((new MailSend($ticket->cust->email, 'customer_sendemail_whenticketclosed', $ticketData)));
                    if ($ccemailsend->ccemails != null) {
                        dispatch((new MailSend($ccemailsend->ccemails, 'CCmail_sendemail_whenticketclosed', $ticketData)));
                    }
                    /**** End Close Ticket mail and notificaton ****/
                } catch (\Exception $e) {
                    if (setting('ticketrating') == 'on') {

                        return redirect()->back()->with("success", lang('The response to the ticket was successful.', 'alerts'));
                    } else {

                        $ratingtoken = Ratingtoken::create([

                            'token' => str_random(64),
                            'ticket_id' => $ticket->id,
                        ]);

                        return redirect()->route('guest.rating', $ratingtoken->token);
                    }
                }

                if (setting('ticketrating') == 'on') {

                    return redirect()->back()->with("success", lang('The response to the ticket was successful.', 'alerts'));
                } else {

                    $ratingtoken = Ratingtoken::create([

                        'token' => str_random(64),
                        'ticket_id' => $ticket->id,
                    ]);

                    return redirect()->route('guest.rating', $ratingtoken->token);
                }
            } else {

                $tickethistory = new tickethistory();
                $tickethistory->ticket_id = $ticket->id;

                $tickethistory->ticketnote = $ticket->ticketnote->isNotEmpty();
                $tickethistory->overduestatus = $ticket->overduestatus;
                $tickethistory->status = $ticket->status;
                $tickethistory->currentAction = 'Responded';
                $tickethistory->username = $comment->cust->username;
                $tickethistory->type = $comment->cust->userType;

                $tickethistory->save();
                $hashids = app(Hashids::class);
                $ticket->encrypted_id = $hashids->encode($ticket->id);

                if ($ticket->cust->userType == 'Guest') {
                    $ticketData = [
                        'ticket_username' => $ticket->cust->username,
                        'ticket_title' => $ticket->subject,
                        'ticket_id' => $ticket->ticket_id,
                        'ticket_status' => $ticket->status,
                        'ticket_description' => $ticket->message,
                        'comment' => $comment->comment,
                        'ratinglink' => route('guest.rating', encrypt($ticket->ticket_id)),
                        'ticket_customer_url' => route('guest.ticketdetailshow',  $ticket->encrypted_id),
                        'ticket_admin_url' => url('/admin/ticket-view/' .  $ticket->encrypted_id),
                        'ticket_category' => $ticket->category ? $ticket->category->name : null,
                        'status' => $ticket->status
                    ];
                }
                if ($ticket->cust->userType == 'Customer') {
                    $ticketData = [
                        'ticket_username' => $ticket->cust->username,
                        'ticket_title' => $ticket->subject,
                        'ticket_id' => $ticket->ticket_id,
                        'ticket_status' => $ticket->status,
                        'comment' => $comment->comment,
                        'ticket_description' => $ticket->message,
                        'ratinglink' => route('guest.rating', encrypt($ticket->ticket_id)),
                        'ticket_customer_url' => route('loadmore.load_data',  $ticket->encrypted_id),
                        'ticket_admin_url' => url('/admin/ticket-view/' .  $ticket->encrypted_id),
                        'ticket_category' => $ticket->category ? $ticket->category->name : null,
                        'status' => $ticket->status
                    ];
                }

                try {

                    // $today = Carbon::today();
                    // $holidays = Holiday::whereDate('startdate', '<=', $today)->whereDate('enddate', '>=', $today)->where('status','1')->get();

                    $today = Carbon::today();
                    $activeHolidays = Holiday::where('status', 1)->get();
                    $now = now();
                    $currentYear = $now->year;
                    $holidays = $activeHolidays->filter(function ($holiday) use ($now, $currentYear) {
                        $holidayStart = Carbon::parse($holiday->startdate);
                        $holidayEnd = Carbon::parse($holiday->enddate);
                        if ($holiday->is_fixed) {
                            return $holidayStart->month === $now->month && $holidayStart->day === $now->day;
                        }
                        return $holidayStart->isSameDay($now);
                    });

                    if ($holidays->isNotEmpty() && setting('24hoursbusinessswitch') != 'on') {
                        dispatch((new MailSend($ticket->cust->email, 'customer_send_ticket_created_that_holiday_or_announcement', $ticketData)));
                        if ($ccemailsend->ccemails != null) {
                            dispatch((new MailSend($ccemailsend->ccemails, 'customer_send_ticket_created_that_holiday_or_announcement', $ticketData)));
                        }
                    } else {
                        dispatch((new MailSend($ticket->cust->email, 'customer_send_ticket_reply', $ticketData)));
                        if ($ccemailsend->ccemails != null) {
                            dispatch((new MailSend($ccemailsend->ccemails, 'customer_send_ticket_reply', $ticketData)));
                        }
                    }

                    /** Reply ticket notification and mail**/
                    if ($ticket->lastreply_mail == null) {
                        if ($ticket->category) {
                            $notificationcat = $ticket->category->groupscategoryc()->get();
                            $groupIds = $notificationcat->pluck('group_id')->toArray();
                            $groupstatus = false;
                            foreach ($groupIds as $groupid) {
                                $groupexist = Groups::where('groupstatus', '1')->find($groupid);
                                if ($groupexist) {
                                    $groupstatus = true;
                                }
                            }

                            $icc = array();

                            if ($groupstatus) {

                                foreach ($notificationcat as $igc) {
                                    $groups = $igc->groupsc()
                                        ->where('groupstatus', 1)
                                        ->with('groupsuser')
                                        ->get();

                                    foreach ($groups as $group) {
                                        $users = $group->groupsuser;

                                        foreach ($users as $user) {
                                            $icc[] = $user->users_id;
                                        }
                                    }
                                }


                                if (!$icc) {
                                    $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                    foreach ($admins as $admin) {
                                        if ($admin->getRoleNames()[0] == 'superadmin') {
                                            $admin->notify(new TicketCreateNotifications($ticket));
                                            if ($admin->usetting->emailnotifyon == 1) {
                                                dispatch((new MailSend($admin->email, 'admin_send_email_ticket_reply', $ticketData)));
                                            }
                                        }
                                    }
                                } else {
                                    if ($ticket->myassignuser) {
                                        $assignee = $ticket->ticketassignmutliples;
                                        foreach ($assignee as $assignees) {
                                            $user = User::where('id', $assignees->toassignuser_id)->where('status', 1)->get();
                                            foreach ($user as $users) {
                                                if ($users->id == $assignees->toassignuser_id) {
                                                    $users->notify(new TicketCreateNotifications($ticket));
                                                    if ($users->usetting->emailnotifyon == 1) {
                                                        dispatch((new MailSend($users->email, 'admin_send_email_ticket_reply', $ticketData)));
                                                    }
                                                }
                                            }
                                        }
                                    } else if ($ticket->selfassignuser_id) {
                                        $self = User::where('status', 1)->findOrFail($ticket->selfassignuser_id);
                                        $self->notify(new TicketCreateNotifications($ticket));
                                        if ($self->usetting->emailnotifyon == 1) {
                                            dispatch((new MailSend($self->email, 'admin_send_email_ticket_reply', $ticketData)));
                                        }
                                    } else if ($icc) {
                                        $user = User::whereIn('id', $icc)->where('status', 1)->get();
                                        foreach ($user as $users) {
                                            $users->notify(new TicketCreateNotifications($ticket));
                                            if ($users->usetting->emailnotifyon == 1) {
                                                dispatch((new MailSend($users->email, 'admin_send_email_ticket_reply', $ticketData)));
                                            }
                                        }
                                        $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                        foreach ($admins as $admin) {
                                            if ($admin->getRoleNames()[0] == 'superadmin') {
                                                $admin->notify(new TicketCreateNotifications($ticket));
                                                if ($admin->usetting->emailnotifyon == 1) {
                                                    dispatch((new MailSend($admin->email, 'admin_send_email_ticket_reply', $ticketData)));
                                                }
                                            }
                                        }
                                    } else {
                                        $users = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                        foreach ($users as $user) {
                                            $user->notify(new TicketCreateNotifications($ticket));
                                            if ($user->usetting->emailnotifyon == 1) {
                                                dispatch((new MailSend($user->email, 'admin_send_email_ticket_reply', $ticketData)));
                                            }
                                        }
                                    }
                                }
                            } else {
                                if ($ticket->myassignuser) {
                                    $assignee = $ticket->ticketassignmutliples;
                                    foreach ($assignee as $assignees) {
                                        $user = User::where('id', $assignees->toassignuser_id)->where('status', 1)->get();
                                        foreach ($user as $users) {
                                            if ($users->id == $assignees->toassignuser_id) {
                                                $users->notify(new TicketCreateNotifications($ticket));
                                                if ($users->usetting->emailnotifyon == 1) {
                                                    dispatch((new MailSend($users->email, 'admin_send_email_ticket_reply', $ticketData)));
                                                }
                                            }
                                        }
                                    }
                                } else if ($ticket->selfassignuser_id) {
                                    $self = User::where('status', 1)->findOrFail($ticket->selfassignuser_id);
                                    $self->notify(new TicketCreateNotifications($ticket));
                                    if ($self->usetting->emailnotifyon == 1) {
                                        dispatch((new MailSend($self->email, 'admin_send_email_ticket_reply', $ticketData)));
                                    }
                                } else {
                                    foreach (usersdata() as $user) {
                                        $user->notify(new TicketCreateNotifications($ticket));
                                        if ($user->usetting->emailnotifyon == 1) {
                                            dispatch((new MailSend($user->email, 'admin_send_email_ticket_reply', $ticketData)));
                                        }
                                    }
                                }
                            }
                        } else {
                            foreach (usersdata() as $admin) {
                                $admin->notify(new TicketCreateNotifications($ticket));
                                if ($admin->usetting->emailnotifyon == 1) {
                                    dispatch((new MailSend($admin->email, 'admin_send_email_ticket_reply', $ticketData)));
                                }
                            }
                        }
                    }

                    if ($ticket->lastreply_mail != null) {
                        if ($ticket->category) {
                            $notificationcat = $ticket->category->groupscategoryc()->get();
                            $groupIds = $notificationcat->pluck('group_id')->toArray();
                            $groupstatus = false;
                            foreach ($groupIds as $groupid) {
                                $groupexist = Groups::where('groupstatus', '1')->find($groupid);
                                if ($groupexist) {
                                    $groupstatus = true;
                                }
                            }

                            $icc = array();

                            if ($groupstatus) {

                                foreach ($notificationcat as $igc) {
                                    $groups = $igc->groupsc()
                                        ->where('groupstatus', 1)
                                        ->with('groupsuser')
                                        ->get();

                                    foreach ($groups as $group) {
                                        $users = $group->groupsuser;

                                        foreach ($users as $user) {
                                            $icc[] = $user->users_id;
                                        }
                                    }
                                }


                                if (!$icc) {
                                    $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                    foreach ($admins as $admin) {
                                        if ($admin->getRoleNames()[0] == 'superadmin') {
                                            $admin->notify(new TicketCreateNotifications($ticket));
                                            if ($admin->usetting->emailnotifyon == 1) {
                                                dispatch((new MailSend($admin->email, 'admin_send_email_ticket_reply', $ticketData)));
                                            }
                                        }
                                    }
                                } else {
                                    if ($ticket->myassignuser_id) {
                                        $assignee = $ticket->ticketassignmutliples;
                                        foreach ($assignee as $assignees) {
                                            $user = User::where('status', 1)->where('id', $assignees->toassignuser_id)->get();
                                            foreach ($user as $users) {
                                                if ($users->id == $assignees->toassignuser_id) {
                                                    $users->notify(new TicketCreateNotifications($ticket));
                                                    if ($users->usetting->emailnotifyon == 1) {
                                                        dispatch((new MailSend($users->email, 'admin_send_email_ticket_reply', $ticketData)));
                                                    }
                                                }
                                            }
                                        }
                                    } else if ($ticket->selfassignuser_id) {

                                        $self = User::where('status', 1)->findOrFail($ticket->selfassignuser_id);
                                        $self->notify(new TicketCreateNotifications($ticket));
                                        if ($self->usetting->emailnotifyon == 1) {
                                            dispatch((new MailSend($self->email, 'admin_send_email_ticket_reply', $ticketData)));
                                        }
                                    } else if ($icc) {
                                        $user = User::whereIn('id', $icc)->where('status', 1)->get();
                                        foreach ($user as $users) {
                                            $users->notify(new TicketCreateNotifications($ticket));
                                            if ($users->usetting->emailnotifyon == 1) {
                                                dispatch((new MailSend($users->email, 'admin_send_email_ticket_reply', $ticketData)));
                                            }
                                        }
                                        $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                        foreach ($admins as $admin) {
                                            if ($admin->getRoleNames()[0] == 'superadmin') {
                                                $admin->notify(new TicketCreateNotifications($ticket));
                                                if ($admin->usetting->emailnotifyon == 1) {
                                                    dispatch((new MailSend($admin->email, 'admin_send_email_ticket_reply', $ticketData)));
                                                }
                                            }
                                        }
                                    } else {
                                        $users = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                                        foreach ($users as $user) {
                                            $user->notify(new TicketCreateNotifications($ticket));
                                            if ($user->usetting->emailnotifyon == 1) {
                                                dispatch((new MailSend($user->email, 'admin_send_email_ticket_reply', $ticketData)));
                                            }
                                        }
                                    }
                                }
                            } else {
                                if ($ticket->myassignuser) {
                                    $assignee = $ticket->ticketassignmutliples;
                                    foreach ($assignee as $assignees) {
                                        $user = User::where('id', $assignees->toassignuser_id)->where('status', 1)->get();
                                        foreach ($user as $users) {
                                            if ($users->id == $assignees->toassignuser_id) {
                                                $users->notify(new TicketCreateNotifications($ticket));
                                                if ($users->usetting->emailnotifyon == 1) {
                                                    dispatch((new MailSend($users->email, 'admin_send_email_ticket_reply', $ticketData)));
                                                }
                                            }
                                        }
                                    }
                                } else if ($ticket->selfassignuser_id) {
                                    $self = User::where('status', 1)->findOrFail($ticket->selfassignuser_id);
                                    $self->notify(new TicketCreateNotifications($ticket));
                                    if ($self->usetting->emailnotifyon == 1) {
                                        dispatch((new MailSend($self->email, 'admin_send_email_ticket_reply', $ticketData)));
                                    }
                                } else {
                                    foreach (usersdata() as $user) {
                                        $user->notify(new TicketCreateNotifications($ticket));
                                        if ($user->usetting->emailnotifyon == 1) {
                                            dispatch((new MailSend($user->email, 'admin_send_email_ticket_reply', $ticketData)));
                                        }
                                    }
                                }
                            }
                        } else {
                            $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')
                                ->leftJoin('groups', 'groups.id', 'groups_users.groups_id')
                                ->select('users.id', 'users.firstname', 'users.lastname', 'users.email', 'groups.groupname', 'groups.groupstatus')
                                ->where('users.status', 1)
                                ->where(function ($query) {
                                    $query->whereNull('groups_users.groups_id')
                                        ->orWhere('groups.groupstatus', '!=', 1);
                                })
                                ->get();
                            foreach ($admins as $admin) {
                                $admin->notify(new TicketCreateNotifications($ticket));
                                if ($admin->usetting->emailnotifyon == 1) {
                                    dispatch((new MailSend($admin->email, 'admin_send_email_ticket_reply', $ticketData)));
                                }
                            }
                        }
                    }


                    /** End Reply ticket notification and mails **/
                } catch (\Exception $e) {
                    return redirect()->back()->with("success", lang('The response to the ticket was successful.', 'alerts'));
                }

                return redirect()->back()->with("success", lang('The response to the ticket was successful.', 'alerts'));
            }
        }
    }

    public function envatoverify(Request $request)
    {
        if ($request->data) {

            $apidatatoken = APIData::first();

            $envato_license = $request->data;

            $url = "https://api.envato.com/v3/market/author/sale?code=" . $envato_license;
            $curl = curl_init($url);

            $personal_api_token = $apidatatoken != null ? $apidatatoken->envatoapitoken : '';

            /*Correct header for the curl extension*/
            $header = array();
            $header[] = 'Authorization: Bearer ' . $personal_api_token;
            $header[] = 'User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10.11; rv:41.0) Gecko/20100101 Firefox/41.0';
            $header[] = 'timeout: 20';
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($curl, CURLOPT_HTTPHEADER, $header);

            /*Connect to the API, and get values from there*/
            $envatoCheck = curl_exec($curl);
            curl_close($curl);

            $envatoCheck = json_decode($envatoCheck);

            $firstone = isset($envatoCheck->error);

            $secondone = null;
            if ($firstone == true) {
                $personal_api_token = $apidatatoken != null ? $apidatatoken->envatoapitokensecond : '';
                $header = array();
                $header[] = 'Authorization: Bearer ' . $personal_api_token;
                $header[] = 'User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10.11; rv:41.0) Gecko/20100101 Firefox/41.0';
                $header[] = 'timeout: 100';
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
                $envatoCheck = curl_exec($curl);
                curl_close($curl);
                $envatoCheck = json_decode($envatoCheck);
                if (isset($envatoCheck->error)) {
                    $secondone = true;
                }
            }

            if ($secondone == true) {
                $personal_api_token = $apidatatoken != null ? $apidatatoken->envatoapitokenthird : '';
                $header = array();
                $header[] = 'Authorization: Bearer ' . $personal_api_token;
                $header[] = 'User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10.11; rv:41.0) Gecko/20100101 Firefox/41.0';
                $header[] = 'timeout: 100';
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
                $envatoCheck = curl_exec($curl);
                curl_close($curl);

                $envatoCheck = json_decode($envatoCheck);
            }

            /*Variable request from the API*/
            $date = new \DateTime(isset($envatoCheck->supported_until) ? $envatoCheck->supported_until : false);
            $support_date = $date->format('Y-m-d H:i:s');
            $sold = new \DateTime(isset($envatoCheck->sold_at) ? $envatoCheck->sold_at : false);
            $sold_at = $sold->format('Y-m-d H:i:s');
            $buyer = (isset($envatoCheck->buyer) ? $envatoCheck->buyer : false);
            $license = (isset($envatoCheck->license) ? $envatoCheck->license : false);
            $count = (isset($envatoCheck->purchase_count) ? $envatoCheck->purchase_count : false);
            $support_amount = (isset($envatoCheck->support_amount) ? $envatoCheck->support_amount : false);
            $item = (isset($envatoCheck->item->previews->landscape_preview->landscape_url) ? $envatoCheck->item->previews->landscape_preview->landscape_url : false);

            /*If Purchase code exists, But Purchase ended*/
            if (isset($envatoCheck->item->name) && (date('Y-m-d H:i:s') >= $support_date)) {
                return response()->json(['valid' => 'expried', 'message' => lang('The purchase code has been verified, but your product support has expired.', 'alerts'), 'key' => $envato_license, 'name' => $envatoCheck->item->name]);
            }

            /*If Purchase code exists, display client information*/
            if (isset($envatoCheck->item->name) && (date('Y-m-d H:i:s') < $support_date)) {
                return response()->json(['valid' => 'true', 'message' => lang('The purchase code has been validated and is supported.', 'alerts'), 'key' => $envato_license, 'name' => $envatoCheck->item->name]);
            }

            /*If Purchase Code doesn't exist,*/
            if (!isset($envatoCheck->item->name)) {
                return response()->json(['valid' => 'false', 'message' => lang('The Purchase Code is invalid.', 'alerts')]);
            }
        }
    }
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function rating($ticket_id)
    {
        $ratingticket = Ratingtoken::where('token', $ticket_id)->first();
        if (!$ratingticket) {

            return redirect('/')->with("error", lang('Your rating has already been submitted.', 'alerts'));
        }
        $ticket = Ticket::where('id', $ratingticket->ticket_id)->first();

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $socialAuthSettings = SocialAuthSetting::first();
        $data['socialAuthSettings'] = $socialAuthSettings;

        $rating = $ticket->comments()->whereNotNull('user_id')->get();
        $comment = Comment::select('user_id')->where('ticket_id', $ticket->id)->distinct()->get();

        if ($rating->isEmpty()) {
            return redirect()->back()->with("success", lang('Your comment has be submitted.', 'alerts'));
        } else {
            return view('guestticket.rating', compact('ticket', 'title', 'comment', 'footertext'))->with($data);
        }
    }

    /// rating system ///
    public function star5($id)
    {

        $user = User::with('usetting')->findorFail($id);
        $user->usetting->increment('star5');
        $user->usetting->update();

        return redirect('/')->with('success', lang('Thank you for rating us.', 'alerts'));
    }

    public function star4($id)
    {

        $user = User::with('usetting')->findorFail($id);
        $user->usetting->increment('star4');
        $user->usetting->update();

        return redirect('/')->with('success', lang('Thank you for rating us.', 'alerts'));
    }

    public function star3($id)
    {

        $user = User::with('usetting')->findorFail($id);
        $user->usetting->increment('star3');
        $user->usetting->update();

        return redirect('/')->with('success', lang('Thank you for rating us.', 'alerts'));
    }

    public function star2($id)
    {

        $user = User::with('usetting')->findorFail($id);
        $user->usetting->increment('star2');
        $user->usetting->update();

        return redirect('/')->with('success', lang('Thank you for rating us.', 'alerts'));
    }

    public function star1($id)
    {

        $user = User::with('usetting')->findorFail($id);
        $user->usetting->increment('star1');
        $user->usetting->update();

        return redirect('/')->with('success', lang('Thank you for rating us.', 'alerts'));
    }

    public function ticketrating(Request $req)
    {
        $req->validate([
            'ratingticket' => 'required',
        ]);

        $ticketfinding = Ticket::find($req->ticket_id);

        $ratingticket = Userrating::where('ticket_id', $req->ticket_id)->first();

        if ($ratingticket) {
            $ratingticket->ratingstar = $req->ratingticket;
            $ratingticket->ratingcomment = $req->ratingcomment;
            $ratingticket->update();

            $employeeratingloop = Employeerating::where('urating_id', $ratingticket->id)->get();
            foreach ($employeeratingloop as $employeeratings) {
                $employeeratings->delete();
            }

            $commentsfind = $ticketfinding->comments()->where('user_id', '!=', null)->distinct()->get();


            $processedUserIds = [];

            foreach ($commentsfind as $commentfinds) {

                if (!in_array($commentfinds->user_id, $processedUserIds) && $commentfinds->user_id != 741741741) {

                    $employeerating = new Employeerating();
                    $employeerating->urating_id = $ratingticket->id;
                    $employeerating->rating = $ratingticket->ratingstar;
                    $employeerating->user_id = $commentfinds->user_id;
                    $employeerating->save();

                    $userdata = User::find($employeerating->user_id);
                    $emaildata = [
                        'ticket_username' => $userdata->name,
                        'username' => $ticketfinding->cust->username,
                        'rating_value' => $ratingticket->ratingstar,
                        'comment' => $ratingticket->ratingcomment,
                        'ticket_admin_url' => route('admin.reports.employeedetails', $userdata->id),
                    ];

                    dispatch((new MailSend($userdata->email, 'send_email_admin_panel_users_when_customer_given_rating', $emaildata)));

                    $processedUserIds[] = $commentfinds->user_id;
                }
            }
        } else {

            $ticketrating = new Userrating();
            $ticketrating->ticket_id = $req->ticket_id;
            $ticketrating->ratingstar = $req->ratingticket;
            $ticketrating->ratingcomment = $req->ratingcomment;
            $ticketrating->cust_id = $ticketfinding->cust->id;
            $ticketrating->save();

            $ticketsfind = Ticket::where('id', $req->ticket_id)->first();
            $commentsfind = $ticketsfind->comments()->where('user_id', '!=', null)->distinct()->get();

            $processedUserIds = []; // Create an array to keep track of processed user IDs.

            foreach ($commentsfind as $commentfinds) {
                // Check if the user ID has already been processed.
                if (!in_array($commentfinds->user_id, $processedUserIds) && $commentfinds->user_id != 741741741) {
                    $employeerating = new Employeerating();
                    $employeerating->urating_id = $ticketrating->id;
                    $employeerating->rating = $ticketrating->ratingstar;
                    $employeerating->user_id = $commentfinds->user_id;
                    $employeerating->save();
                    $userdata = User::find($employeerating->user_id);
                    $emaildata = [
                        'ticket_username' => $userdata->name,
                        'username' => $ticketfinding->cust->username,
                        'rating_value' => $ticketrating->ratingstar,
                        'comment' => $ticketrating->ratingcomment,
                        'ticket_admin_url' => route('admin.reports.employeedetails', $userdata->id),
                    ];

                    dispatch((new MailSend($userdata->email, 'send_email_admin_panel_users_when_customer_given_rating', $emaildata)));

                    $processedUserIds[] = $commentfinds->user_id;
                }
            }
        }

        $ratingticketdelete = Ratingtoken::where('ticket_id', $req->ticket_id)->first();
        $ratingticketdelete->delete();
        return redirect()->route('home')->with('success', lang('Thank you for rating us.', 'alerts'));
    }
    /// end rating system ///

    public function imagedestroy($id)
    {
        //For Deleting Users
        $commentss = Media::findOrFail($id);
        $commentss->delete();
        return response()->json([
            'success' => lang('The image has been deleted successfully!', 'alerts'),
        ]);
    }

    public function close(Request $request, $ticket_id)
    {
        $ticket_id = decrypt($ticket_id);
        $ticket = Ticket::where('ticket_id', $ticket_id)->firstOrFail();

        $ticket->status = 'Re-Open';

        $ticket->update();

        $tickethistory = new tickethistory();
        $tickethistory->ticket_id = $ticket->id;

        $tickethistory->ticketnote = $ticket->ticketnote->isNotEmpty();
        $tickethistory->overduestatus = $ticket->overduestatus;
        $tickethistory->status = $ticket->status;
        $tickethistory->currentAction = 'Closed';
        $tickethistory->username = $ticket->cust->username;
        $tickethistory->type = $ticket->cust->userType;

        $tickethistory->save();
        if (webSocketCheck()) {
            try {
                event(new \App\Events\ConversationEvent(null, null, null, null, null, 'tableDataReload', null, null));
                event(new \App\Events\ConversationEvent(dbname(), null, 'Reopen', null, null, null, 'customer', $ticket->ticket_id, null));
            } catch (\Exception $e) {
            }
        }

        // Create a New ticket reply
        if ($ticket->category) {
            $notificationcat = $ticket->category->groupscategoryc()->get();
            $groupIds = $notificationcat->pluck('group_id')->toArray();
            $groupstatus = false;
            foreach ($groupIds as $groupid) {
                $groupexist = Groups::where('groupstatus', '1')->find($groupid);
                if ($groupexist) {
                    $groupstatus = true;
                }
            }

            $icc = array();

            if ($groupstatus) {

                foreach ($notificationcat as $igc) {
                    $groups = $igc->groupsc()
                        ->where('groupstatus', 1)
                        ->with('groupsuser')
                        ->get();

                    foreach ($groups as $group) {
                        $users = $group->groupsuser;

                        foreach ($users as $user) {
                            $icc[] = $user->users_id;
                        }
                    }
                }

                if (!$icc) {
                    $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->get();
                    foreach ($admins as $admin) {
                        $admin->notify(new TicketCreateNotifications($ticket));
                    }
                } else {

                    $user = User::whereIn('id', $icc)->get();
                    foreach ($user as $users) {
                        $users->notify(new TicketCreateNotifications($ticket));
                    }
                    $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->get();
                    foreach ($admins as $admin) {
                        $admin->notify(new TicketCreateNotifications($ticket));
                    }
                }
            } else {
                foreach (usersdata() as $admin) {
                    $admin->notify(new TicketCreateNotifications($ticket));
                }
            }
        }

        // Notification category Empty
        if (!$ticket->category) {
            $admins = User::get();
            foreach ($admins as $admin) {
                $admin->notify(new TicketCreateNotifications($ticket));
            }
        }

        $ccemailsend = CCMAILS::where('ticket_id', $ticket->id)->first();

        $hashids = app(Hashids::class);
        $ticket->encrypted_id = $hashids->encode($ticket->id);


        $ticketData = [
            'ticket_title' => $ticket->subject,
            'ticket_description' => $ticket->message,
            'ticket_id' => $ticket->ticket_id,
            'ticket_username' => $ticket->cust->username,
            'ticket_status' => $ticket->status,
            'ticket_customer_url' => route('guest.ticketdetailshow', $ticket->encrypted_id),
            'ticket_admin_url' => url('/admin/ticket-view/' . $ticket->encrypted_id),
            'ticket_category' => $ticket->category ? $ticket->category->name : null,
            'status' => $ticket->status
        ];

        try {

            if ($ticket->category) {
                $notificationcat = $ticket->category->groupscategoryc()->get();
                $groupIds = $notificationcat->pluck('group_id')->toArray();
                $groupstatus = false;
                foreach ($groupIds as $groupid) {
                    $groupexist = Groups::where('groupstatus', '1')->find($groupid);
                    if ($groupexist) {
                        $groupstatus = true;
                    }
                }

                $icc = array();

                if ($groupstatus) {

                    foreach ($notificationcat as $igc) {
                        $groups = $igc->groupsc()
                            ->where('groupstatus', 1)
                            ->with('groupsuser')
                            ->get();

                        foreach ($groups as $group) {
                            $users = $group->groupsuser;

                            foreach ($users as $user) {
                                $icc[] = $user->users_id;
                            }
                        }
                    }
                    if (!$icc) {
                        $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->get();
                        foreach ($admins as $admin) {
                            if ($admin->usetting->emailnotifyon == 1) {
                                dispatch((new MailSend($admin->email, 'admin_sendemail_whenticketreopen', $ticketData)));
                            }
                        }
                    } else {

                        $user = User::whereIn('id', $icc)->where('status', 1)->get();
                        foreach ($user as $users) {
                            if ($users->usetting->emailnotifyon == 1) {
                                dispatch((new MailSend($users->email, 'admin_sendemail_whenticketreopen', $ticketData)));
                            }
                        }
                        $admins = User::leftJoin('groups_users', 'groups_users.users_id', 'users.id')->whereNull('groups_users.groups_id')->whereNull('groups_users.users_id')->where('users.status', 1)->get();
                        foreach ($admins as $admin) {
                            if ($admin->usetting->emailnotifyon == 1) {
                                dispatch((new MailSend($admin->email, 'admin_sendemail_whenticketreopen', $ticketData)));
                            }
                        }
                    }
                } else {
                    foreach (usersdata() as $admin) {
                        if ($admin->usetting->emailnotifyon == 1) {
                            dispatch((new MailSend($admin->email, 'admin_sendemail_whenticketreopen', $ticketData)));
                        }
                    }
                }
            }
            if (!$ticket->category) {
                $admins = User::where('users.status', 1)->get();
                foreach ($admins as $admin) {
                    if ($admin->usetting->emailnotifyon == 1) {
                        dispatch((new MailSend($admin->email, 'admin_sendemail_whenticketreopen', $ticketData)));
                    }
                }
            }

            dispatch((new MailSend($ticket->cust->email, 'customer_send_ticket_reopen', $ticketData)));
            if ($ccemailsend->ccemails != null) {
                dispatch((new MailSend($ccemailsend->ccemails, 'customer_send_ticket_reopen', $ticketData)));
            }
        } catch (\Exception $e) {
            return redirect()->back()->with("success", lang('The ticket has been successfully reopened.', 'alerts'));
        }

        return redirect()->back()->with("success", lang('The ticket has been successfully reopened.', 'alerts'));
    }

    public function emailsvalidateguest(Request $request)
    {
        if (setting('GUEST_TICKET') == 'yes') {
            $request->validate([
                'email' => 'required|max:255|indisposable|email',

            ]);

            $customers = Customer::where('email', $request->email)->first();
            if ($customers != null && $customers->userType == 'Customer') {
                return response()->json(['message' => 'alreadyregistered', 'error' => lang('Email is already registered, Please login to create a Ticket.', 'alerts')], 200);
            }

            // Guest restrict to create tickets based on allowed to create.
            if (setting('RESTRICT_TO_CREATE_TICKET') == 'on' && setting('MAXIMUM_ALLOW_TICKETS') > 0) {
                if ($customers != null && $customers->tickets != null) {
                    $star1 = now()->subHour(setting('MAXIMUM_ALLOW_HOURS'));
                    $star2 = now();
                    $latestReplies = $customers->tickets()->whereBetween('created_at', [$star1, $star2])->get()->take(setting('MAXIMUM_ALLOW_TICKETS'));
                    if ($latestReplies->isNotEmpty()) {
                        $totalcount = 0;
                        foreach ($latestReplies as $comment) {
                            if ($comment->user_id !== null) {
                                $totalcount++;
                            }
                        }
                        if (!$totalcount) {
                            $currentTcikets = $latestReplies->last();
                            $data['difference'] = $currentTcikets->created_at->addHour(setting('REPLY_ALLOW_IN_HOURS'))->diffForHumans(now());

                            $ticketscount = Ticket::where('cust_id', $customers->id)->whereBetween('created_at', [$star1, $star2])->count();
                            if ($ticketscount >= setting('MAXIMUM_ALLOW_TICKETS')) {
                                return response()->json(['message' => 'ticcreaterestrict', 'error' => lang('Customer has reached maximum allowed tickets to create.', 'alerts')], 200);
                            }
                        }
                    }
                }
            }

            $email = $request->email;
            $completeDomain = substr(strrchr($email, "@"), 1);
            $emaildomainlist = setting('EMAILDOMAIN_LIST');
            $emaildomainlistArray = explode(",", $emaildomainlist);

            if (setting('EMAILDOMAIN_BLOCKTYPE') == 'blockemail') {

                if (setting('EMAILDOMAIN_LIST') == null) {
                    $ticket = $this->createverifyotp($request);
                    return response()->json(['message' => 'createverifyotp', 'success' => lang('An OTP (One Time Password) has been sent to your email ID. Please enter the OTP below to submit your guest ticket', 'alerts')], 200);
                } else {
                    if (in_array($completeDomain, $emaildomainlistArray)) {

                        return response()->json(['message' => 'domainblock', 'error' => lang('Domain is Blocked List', 'alerts')], 200);
                    }
                    $ticket = $this->createverifyotp($request);
                    return response()->json(['message' => 'createverifyotp', 'success' => lang('An OTP (One Time Password) has been sent to your email ID. Please enter the OTP below to submit your guest ticket.', 'alerts')], 200);
                }
            }

            if (setting('EMAILDOMAIN_BLOCKTYPE') == 'allowemail') {
                if (setting('EMAILDOMAIN_LIST') == null) {
                    $ticket = $this->createverifyotp($request);
                    return response()->json(['message' => 'createverifyotp', 'success' => lang('An OTP (One Time Password) has been sent to your email ID. Please enter the OTP below to submit your guest ticket.', 'alerts')], 200);
                } else {
                    if (in_array($completeDomain, $emaildomainlistArray)) {
                        $ticket = $this->createverifyotp($request);
                        return response()->json(['message' => 'createverifyotp', 'success' => lang('An OTP (One Time Password) has been sent to your email ID. Please enter the OTP below to submit your guest ticket.', 'alerts')], 200);
                    }
                    return response()->json(['message' => 'domainblock', 'error' => lang('Domain is Blocked List', 'alerts')], 200);
                }
            }
        } else {
            return response()->json(['message' => 'accessdenied', 'error' => lang('you are not autherised to this ticket create', 'alerts')], 500);
        }
    }

    private function createverifyotp($request)
    {

        $guest = VerifyOtp::where('type', 'guestotp')->where('cust_id', $request->email)->first();

        if ($guest) {
            $guest->otp = rand(100000, 999999);
            $guest->update();
            if ($request->session()->has('sessionemail')) {
                $request->session()->forget('sessionemail');
            }
            $request->session()->put('sessionemail', $guest->cust_id);
            $guestticket = [

                'guestotp' => $guest->otp,
                'guestemail' => $guest->cust_id,
                'guestname' => 'Guest',
            ];

            try {

                dispatch((new MailSend($guest->cust_id, 'guestticket_email_verification', $guestticket)));
            } catch (\Exception $e) {

                // return response()->json(['success' => lang('Please check your Email', 'alerts'), 'email' => 'exists']);
            }
        }
        if (!$guest) {
            $verifyOtp = VerifyOtp::create([
                'cust_id' => $request->email,
                'otp' => rand(100000, 999999),
                'type' => 'guestotp',
            ]);

            if ($request->session()->has('sessionemail')) {
                $request->session()->forget('sessionemail');
            }
            $request->session()->put('sessionemail', $verifyOtp->cust_id);
            $guestticket = [

                'guestotp' => $verifyOtp->otp,
                'guestemail' => $verifyOtp->cust_id,
                'guestname' => 'Guest',
            ];

            try {

                dispatch((new MailSend($verifyOtp->cust_id, 'guestticket_email_verification', $guestticket)));
            } catch (\Exception $e) {

                // return response()->json(['success' => lang('Please check your Email', 'alerts'), 'email' => 'exists']);
            }
        }
    }

    public function verifyotp(Request $request)
    {
        if ($request->session()->has('sessionemail')) {
            $emailvalidate = $request->session()->get('sessionemail');
        }
        $verify = VerifyOtp::where('type', 'guestotp')->where('otp', $request->otpvalue)->first();
        if ($verify) {
            if ($emailvalidate == $verify->cust_id) {
                $customerfind = Customer::where('email', $verify->cust_id)->first();
                if (!$customerfind) {
                    $guest = Customer::create([

                        'firstname' => '',
                        'lastname' => '',
                        'username' => 'GUEST',
                        'email' => $verify->cust_id,
                        'userType' => 'Guest',
                        'password' => null,
                        'status' => '1',
                        'image' => null,

                    ]);
                    $customersetting = new CustomerSetting();
                    $customersetting->custs_id = $guest->id;
                    $customersetting->save();

                    return response()->json(['success' => lang('verified'), $guest], 200);
                }

                if ($customerfind) {
                    return response()->json(['success' => lang('verified'), $customerfind], 200);
                }
            }
        } else {
            return response()->json(['error' => lang('Invalid OTP')], 200);
        }
        if (!$verify) {
            return response()->json(['error' => lang('Invalid OTP')], 200);
        }
    }

    public function subcategorylist(Request $request)
    {

        $parent_id = $request->cat_id;

        $subcategoriess = Subcategorychild::where('category_id', $parent_id)->get();

        $subcatstatusexisting = null;
        $output = '';
        $myArray = [];

        if ($subcategoriess->isNotEmpty()) {
            $output .= '<option label="select subcategory"></option>';
            foreach ($subcategoriess as $subcats) {
                $sucatss = $subcats->subcatlists()->where('status', '1')->get();
                if ($sucatss->isNotEmpty()) {
                    $subcatstatusexisting = 'statusexisting';
                    foreach ($sucatss as $subcategory) {
                        $myArray[] = $subcategory;
                        $output .= '<option value="' . $subcategory->id . '">' . $subcategory->subcategoryname . '</option>';
                    }
                }
            }
        }

        //projectlist
        $subcategories = Projects::select('projects.*', 'projects_categories.category_id')->join('projects_categories', 'projects_categories.projects_id', 'projects.id')
            ->where('projects_categories.category_id', $parent_id)
            ->get();

        // envato asssign

        $categoryenvato = CategoryEnvato::where('category_id', $parent_id)->get();

        $data = [
            'subCatStatus' => $myArray,
            'subcategoriess' => $output,
            'subcategories' => $subcategories,
            'envatosuccess' => $categoryenvato,
            'subcatstatusexisting' => $subcatstatusexisting,
        ];
        return response()->json(
            $data,
            200
        );
    }

    public function ticketview($id)
    {
        $ticket = Ticket::find($id);
        $data['ticket'] = $ticket;

        $title = Apptitle::first();
        $data['title'] = $title;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $socialAuthSettings = SocialAuthSetting::first();
        $data['socialAuthSettings'] = $socialAuthSettings;

        return view('guestticket.ticketverify')->with($data);
    }

    public function senddataverify(Request $request)
    {
        $ticketid = $request->id;

        $ticketfind = Ticket::find($ticketid);

        $email = $ticketfind->cust->email;

        $guest = VerifyOtp::where('type', 'guestverifyotp')->where('cust_id', $email)->first();

        if (!$guest) {
            $verifyOtp = VerifyOtp::create([
                'cust_id' => $email,
                'otp' => rand(100000, 999999),
                'type' => 'guestverifyotp',
            ]);

            if ($request->session()->has('sessionverifyemail')) {
                $request->session()->forget('sessionverifyemail');
            }
            $request->session()->put('sessionverifyemail', $ticketfind->id);
            session()->put('guestdetailssession',  $ticketfind->id);
            $guestticket = [

                'guestotp' => $verifyOtp->otp,
                'guestemail' => $verifyOtp->cust_id,
                'guestname' => 'Guest',
            ];

            try {

                dispatch((new MailSend($verifyOtp->cust_id, 'guestticket_email_verification_view', $guestticket)));
            } catch (\Exception $e) {

                return response()->json(['success' => lang('Please check your Email', 'alerts'), 'verifyemail' => 'OK']);
            }
            return response()->json(['success' => lang('Please check your Email', 'alerts'), 'verifyemail' => 'OK']);
        }

        if ($guest) {
            $guest->otp = rand(100000, 999999);
            $guest->update();
            if ($request->session()->has('sessionverifyemail')) {
                $request->session()->forget('sessionverifyemail');
            }
            $request->session()->put('sessionverifyemail', $guest->cust_id);
            $guestticket = [

                'guestotp' => $guest->otp,
                'guestemail' => $guest->cust_id,
                'guestname' => 'Guest',
            ];

            try {

                dispatch((new MailSend($guest->cust_id, 'guestticket_email_verification_view', $guestticket)));
            } catch (\Exception $e) {

                return response()->json(['success' => lang('Please check your Email', 'alerts'), 'verifyemail' => 'OK']);
            }
            return response()->json(['success' => lang('Please check your Email', 'alerts'), 'verifyemail' => 'OK']);
        }
    }

    public function verifyguestotp(Request $request)
    {

        $otpverify = $request->otpvalue;

        if ($request->session()->has('sessionemail')) {
            $emailvalidate = $request->session()->get('sessionemail');
        }
        $verify = VerifyOtp::where('type', 'guestverifyotp')->where('otp', $request->otpvalue)->first();
        if ($verify) {
            $hashids = app(Hashids::class);
            $guestticketdetailssession = session()->put('sessionverifyemail', $hashids->decode($request->ticket_id)[0]);
            return response()->json(['success' => lang('Valid OTP')], 200);
        }
        if (!$verify) {
            return response()->json(['error' => lang('Invalid OTP')], 200);
        }
    }

    public function ticketdetailshow($ticket_id)
    {
        // $ticket_id = decrypt($ticket_id);
        $aaa = Str::length($ticket_id);

        $hashids = app(Hashids::class);
        if ($aaa == 20) {
            $ticket_id = $hashids->decode($ticket_id)[0];

            $ticket = Ticket::where('id', $ticket_id)->firstOrFail();
        } elseif ($aaa > 20) {
            $ticket_id = decrypt($ticket_id);
            $ticket = Ticket::where('ticket_id', $ticket_id)->firstOrFail();
        } elseif ($aaa < 20) {
            $ticket = Ticket::where('ticket_id', $ticket_id)->firstOrFail();
        }

        // $ticket = Ticket::where('ticket_id', $ticket_id)->firstOrFail();
        $ticket->encrypted_ticketId = encrypt($ticket->ticket_id);
        $comments = $ticket->comments()->paginate(5);
        $category = $ticket->category;

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $socialAuthSettings = SocialAuthSetting::first();
        $data['socialAuthSettings'] = $socialAuthSettings;

        $verifyUser = VerifyOtp::where('type', 'guestverifyotp')->where('cust_id', $ticket->cust->email)->first();
        if ($verifyUser) {

            $verifyUser->delete();
        }

        $now = now();
        $announcement = announcement::whereDate('enddate', '>=', $now->toDateString())->whereDate('startdate', '<=', $now->toDateString())->get();
        $data['announcement'] = $announcement;

        $announcements = Announcement::whereNotNull('announcementday')->get();
        $data['announcements'] = $announcements;

        // $holidays = Holiday::whereDate('startdate', '<=', $now->toDateString())->whereDate('enddate', '>=', $now->toDateString())->where('status', 1)->get();
        // $data['holidays'] =  $holidays;

        $currentYear = $now->year;
        $activeHolidays = Holiday::where('status', 1)->get();

        // Filter holidays
        $data['holidays'] = $activeHolidays->filter(function ($holiday) use ($now, $currentYear) {
            $holidayStart = Carbon::parse($holiday->startdate);
            $holidayEnd = Carbon::parse($holiday->enddate);

            // Check for fixed holidays (compare only day and month)
            if ($holiday->is_fixed) {
                return $holidayStart->month === $now->month && $holidayStart->day === $now->day;
            }

            // For non-fixed holidays, check the full date
            return $holidayStart->isSameDay($now);
        });

        $data['addspace'] = AddSpace::where('status', 1)->get();

        // customer restrict to reply for the ticket.
        $commentsNull = $ticket->comments()->get();
        if ($commentsNull->all() != null) {

            foreach ($commentsNull as $latestone) {
                if ($latestone->lastseen == null && $latestone->user_id != null) {
                    $latestone->lastseen = now();
                    $latestone->save();
                }
            }
        }

        if (setting('RESTRICT_TO_REPLY_TICKET') == 'on' && $commentsNull->all() != null && setting('MAXIMUM_ALLOW_REPLIES') > 0) {

            $star1 = now()->subHour(setting('REPLY_ALLOW_IN_HOURS'));
            $star2 = now();
            $latestReplies = $ticket->comments()->whereBetween('created_at', [$star1, $star2])->get()->take(setting('MAXIMUM_ALLOW_REPLIES'));
            $totalcount = 0;
            foreach ($latestReplies as $comment) {
                if ($comment->user_id !== null) {
                    $totalcount++;
                }
            }

            if (!$totalcount) {
                $currentTcikets = $latestReplies->last();
                $data['difference'] = $currentTcikets->created_at->addHour(setting('REPLY_ALLOW_IN_HOURS'))->diffForHumans(now());
                $createdcount = $ticket->comments()->where('cust_id', $ticket->cust_id)->whereBetween('created_at', [$star1, $star2])->count();

                if (session()->get('sessionverifyemail') == $ticket_id) {
                    if (request()->page) {
                        $view = view('guestticket.showdataticket', compact('comments', 'createdcount', 'ticket'))->render();
                        return response()->json(['html' => $view]);
                    }
                    return view('guestticket.showticketview', compact('ticket', 'category', 'comments', 'createdcount'))->with($data);
                } else {
                    return redirect()->route('guest.ticketview', $ticket->id);
                }
            }
        }

        $createdcount = '';
        if (session()->get('sessionverifyemail') == $ticket_id) {
            if (request()->page) {
                $view = view('guestticket.showdataticket', compact('comments', 'createdcount', 'ticket'))->render();
                return response()->json(['html' => $view]);
            }
            return view('guestticket.showticketview', compact('ticket', 'category', 'comments', 'createdcount'))->with($data);
        } else {

            return redirect()->route('guest.ticketview', $ticket->id);
        }
    }

    public function notificationalerts()
    {
        if (Auth::check() && Auth::user()) {
            $data = Auth::user()->unreadNotifications()->where('notifiable', '0')->get();
            $dataa = $data->count();
            return response()->json($data, 200);
        }
    }

    public function notificationalertsread()
    {
        if (Auth::check() && Auth::user()) {
            $data = Auth::user()->unreadNotifications()->get();
            foreach ($data as $dataas) {
                $dataas->notifiable = '1';
                $dataas->update();
            }
            return response()->noContent();
        }
    }

    public function notificationsreading()
    {

        $notify = auth()->user()->unreadNotifications()->paginate(2);
        $data['notifys'] = $notify;
        return view('includes.admin.allnotify')->with($data);
    }

    public function badgecount()
    {

        $badgecount = auth()->user()->unreadNotifications->count();
        $data['badgecount'] = $badgecount;
        return view('includes.admin.badgecount')->with($data);
    }

    public function markasreadcount()
    {

        $badgecount = auth()->user()->unreadNotifications->count();
        $data['badgecount'] = $badgecount;
        return view('includes.admin.markasreadcount')->with($data);
    }

    public function allnotifycount()
    {

        $badgecount = auth()->user()->unreadNotifications->count();
        $data['badgecount'] = $badgecount;
        return view('includes.admin.allnotify')->with($data);
    }

    public function cnotificationalerts()
    {

        if (Auth::guard('customer')->check() && Auth::guard('customer')->user()) {
            $data = Auth::guard('customer')->user()->unreadNotifications()->where('notifiable', '0')->get();
            $dataa = $data->count();
            return response()->json($data, 200);
        }
    }

    public function cnotificationalertsread()
    {
        if (Auth::guard('customer')->check() && Auth::guard('customer')->user()) {
            $data = Auth::guard('customer')->user()->unreadNotifications()->get();
            foreach ($data as $dataas) {
                $dataas->notifiable = '1';
                $dataas->update();
            }
            return response()->noContent();
        }
    }

    public function cnotificationsreading()
    {

        $notify = auth()->guard('customer')->user()->unreadNotifications()->paginate(2);
        $data['notifys'] = $notify;
        return view('includes.user.allnotify')->with($data);
    }

    public function cbadgecount()
    {

        $badgecount = auth()->guard('customer')->user()->unreadNotifications->count();
        $data['badgecount'] = $badgecount;
        return view('includes.user.badgecount')->with($data);
    }
    public function cmarkasreadcount()
    {

        $badgecount = auth()->guard('customer')->user()->unreadNotifications->count();
        $data['badgecount'] = $badgecount;
        return view('includes.user.markasreadcount')->with($data);
    }

    public function callnotifycount()
    {

        $badgecount = auth()->guard('customer')->user()->unreadNotifications->count();
        $data['badgecount'] = $badgecount;
        return view('includes.user.allnotify')->with($data);
    }

    public function markallnotify()
    {
        auth()->guard('customer')->user()->unreadNotifications->markAsRead();

        return response()->noContent();
    }


    public function editticketsubject(Request $request)
    {
        $ticket = Ticket::findOrFail($request->ticketId);
        $ticket->subject = $request->editsubject;
        $ticket->message = strip_tags($request->editmessage);
        $ticket->save();

        return response()->json(['ticketdata' => $ticket, 'success' => lang('Updated successfully.', 'alerts')]);
    }

    public function pdfmake($id)
    {
        $id = decrypt($id);
        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $page = Pages::all();
        $data['page'] = $page;

        $showprintticket = Ticket::findOrFail($id);
        $data['showprintticket'] = $showprintticket;

        return view('user.ticket.ticketshowpdf')->with($data);
    }
}
