<?php

namespace App\Http\Controllers\User;

use Str;

use Hash;
use DataTables;
use Carbon\Carbon;
use App\Models\User;
use Hashids\Hashids;
use App\Models\Pages;
use App\Models\Holiday;
use AuthenticatesUsers;
use App\Models\AddSpace;
use App\Models\Apptitle;
use App\Models\Footertext;
use App\Models\Seosetting;
use App\Models\Announcement;
use Illuminate\Http\Request;
use App\Models\Ticket\Ticket;
use App\Models\AllTransactions;
use App\Http\Controllers\Controller;
use App\Models\Setting;
use Illuminate\Notifications\Notification;
use Illuminate\Support\Facades\Auth;

class DashboardController extends Controller
{


    public function userTickets()
    {
        $usertickets = Ticket::query();
        $usertickets = $usertickets->where('cust_id', Auth::guard('customer')->user()->id)->with('category')->get()
            ->sortBy(function ($ticket) {
                return $ticket->overduestatus === 'Overdue' ? 1 : 0;
            })
            ->sortByDesc(function ($ticket) {
                return $ticket->overduestatus === 'Overdue' ? $ticket->created_at : $ticket->updated_at;
            })
            ->values();

            $usertickets = $usertickets->each(function ($ticket)  {

                $hashids = app(Hashids::class);
                $ticket->encrypted_id = $hashids->encode($ticket->id);

                // $ticket->encrypted_id = encrypt($ticket->id);
                // $ticket->encrypted_ticket_id = encrypt($ticket->ticket_id);
                return $ticket;
            });

        $data['usertickets'] = $usertickets;

        if(request()->ajax()){
            return response()->json(['usertickets' => $data['usertickets']]);
        }

        $active = Ticket::where('cust_id', Auth::guard('customer')->user()->id)
            ->whereIn('status', ['New', 'Re-Open', 'Inprogress'])->get();

        $closed = Ticket::where('cust_id', Auth::guard('customer')->user()->id)
            ->where('status', 'Closed')->get();

        $onhold = Ticket::where('cust_id', Auth::guard('customer')->user()->id)
            ->where('status', 'On-Hold')->get();
        $data['onhold'] = $onhold;

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $now = now();
        $announcement = announcement::whereDate('enddate', '>=', $now->toDateString())->whereDate('startdate', '<=', $now->toDateString())->get();
        $data['announcement'] = $announcement;

        $announcements = Announcement::whereNotNull('announcementday')->get();
        $data['announcements'] = $announcements;

        // $holidays = Holiday::whereDate('startdate', '<=', $now->toDateString())->whereDate('enddate', '>=', $now->toDateString())->where('status', '1')->get();
        // $data['holidays'] =  $holidays;

        $now = now();
        $currentYear = $now->year;
        $activeHolidays = Holiday::where('status', 1)->get();


        $data['holidays'] = $activeHolidays->filter(function ($holiday) use ($now, $currentYear) {
            $holidayStart = Carbon::parse($holiday->startdate);
            $holidayEnd = Carbon::parse($holiday->enddate);
            if ($holiday->is_fixed) {
                return $holidayStart->month === $now->month && $holidayStart->day === $now->day;
            }
            return $holidayStart->isSameDay($now);
        });


        // $hasquotations = AllTransactions::where('cust_id', Auth::guard('customer')->user()->id)->exists();
        // $data['hasquotations'] = $hasquotations;

        $quotations = AllTransactions::where('cust_id', Auth::guard('customer')->user()->id) ->whereIn('status', ['pending', 'cancelled'])->latest('created_at')->get();
        $data['quotations'] = $quotations;

        $unviewedQuotations = $quotations->where('cust_viewed', '!=', 1)->where('status','pending');
        $data['unviewedQuotations'] = $unviewedQuotations;

        $invoices = AllTransactions::where('cust_id', Auth::guard('customer')->user()->id)->where('status', 'paid')->get();
        $data['invoices'] = $invoices;

        $refunds = AllTransactions::where('cust_id', Auth::guard('customer')->user()->id)->where('status', ['refunded', 'refundpending', 'refundRejected'])->get();
        $data['refunds'] = $refunds;

        $addspace = AddSpace::where('status',1)->get();

        $customernotify = auth()->guard('customer')->user()->unreadNotifications()->where('data->status', 'mail')->get();

        return view('user.dashboard', compact('usertickets', 'active', 'closed', 'title', 'footertext' ,'addspace'))->with($data);
    }


    public function notify()
    {

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $notifications = auth()->guard('customer')->user()->notifications()->paginate('10')->groupBy(function ($date) {
            return \Carbon\Carbon::parse($date->created_at)->format('Y-m-d');
        });
        $data['notifications'] = $notifications;

        $addspace = AddSpace::where('status',1)->get();

        $quotations = AllTransactions::where('cust_id', Auth::guard('customer')->user()->id) ->whereIn('status', ['pending', 'cancelled'])->latest('created_at')->get();
        $data['quotations'] = $quotations;

        $unviewedQuotations = $quotations->where('cust_viewed', '!=', 1)->where('status','pending');
        $data['unviewedQuotations'] = $unviewedQuotations;


        return view('user.notification', compact('addspace'))->with($data);
    }

    public function markNotification(Request $request)
    {
        auth()->guard('customer')->user()
            ->unreadNotifications
            ->when($request->input('id'), function ($query) use ($request) {
                return $query->where('id', $request->input('id'));
            })
            ->markAsRead();

        return response()->noContent();
    }

    public function Notificationview($id)
    {
        $notification = auth()->guard('customer')->user()->notifications()->where('id', $id)->firstOrFail();
        $data['notifications'] = $notification;

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $post = Pages::all();
        $data['page'] = $post;

        $addspace = AddSpace::where('status',1)->get();

        $quotations = AllTransactions::where('cust_id', Auth::guard('customer')->user()->id) ->whereIn('status', ['pending', 'cancelled'])->latest('created_at')->get();
        $data['quotations'] = $quotations;

        $unviewedQuotations = $quotations->where('cust_viewed', '!=', 1)->where('status','pending');
        $data['unviewedQuotations'] = $unviewedQuotations;



        return view('user.notification.viewnotification')->with($data);
    }

    public function notifystatus(Request $request)
    {
        $status = $request->statusnotify;
        if (is_array($status) && in_array('refund', $status)) {
            // Merge additional statuses
            $status = array_merge($status, ['refundpending', 'refundRejected', 'refunded']);
        }

        $perPage = 10;
        $page = $request->page != null ? $request->page : 0;

        if (!$status) {
            $notifications = auth()->guard('customer')->user()->notifications()->paginate($perPage, ['*'], 'page', $page)
            ->groupBy(function ($date) {
                return \Carbon\Carbon::parse($date->created_at)->format('Y-m-d');
            });
            // $test = false;
        } else {
            $notifications =  auth()->guard('customer')->user()->notifications()->where(function ($query) use ($status) {
                $query->whereIn('data->status', $status)
                    ->orWhere(function ($query) use ($status) {
                        foreach ($status as $stat) {
                            $query->orWhere('data->itemStatus', 'LIKE', "%{$stat}%");
                        }
                    });
            })
            ->paginate($perPage, ['*'], 'page', $page)->groupBy(function ($date) {
                return \Carbon\Carbon::parse($date->created_at)->format('Y-m-d');
            });
            // $test = true;
        }

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $view = view('user.notificationpageinclude', compact('notifications', 'title', 'footertext', 'seopage', 'status', 'perPage', 'page'))->render();
        return response()->json(['html' => $view]);
    }


    public function notifysearch(Request $request)
    {

        $status = $request->notifysearch;

        if ($status) {
            $notifications = auth()->guard('customer')->user()->notifications()->where(function ($query) use ($status) {
                // $keyword = request()->notifysearch;
                $query->where('data->title', 'LIKE', "%{$status}%")
                    ->orwhere('data->status', 'LIKE', "%{$status}%")
                    ->orWhere('data->ticket_id', 'LIKE', "%{$status}%")
                    ->orWhere('data->mailsubject', 'LIKE', "%{$status}%")
                    ->orWhere('data->mailtext', 'LIKE', "%{$status}%");
            })->get()->groupBy(function ($date) {
                return \Carbon\Carbon::parse($date->created_at)->format('Y-m-d');
            });

        } else {
            $notifications =  auth()->guard('customer')->user()->notifications()->paginate()->groupBy(function ($date) {
                return \Carbon\Carbon::parse($date->created_at)->format('Y-m-d');
            });
        }

        $title = Apptitle::first();
        $data['title'] = $title;

        $footertext = Footertext::first();
        $data['footertext'] = $footertext;

        $seopage = Seosetting::first();
        $data['seopage'] = $seopage;

        $view = view('user.notificationpageinclude', compact('notifications', 'title', 'footertext', 'seopage'))->render();
        return response()->json(['html' => $view]);
    }

    public function notifydelete(Request $request)
    {
        $id = $request->id;

        $notificationsdelete = auth()->guard('customer')->user()->notifications()->find($id);
        $notificationsdelete->delete();

        return response()->json(['success' => lang('Deleted Successfully', 'alerts'), 200]);
    }
}
