@extends('layouts.adminmaster')

@section('styles')
    <!-- INTERNAL Data table css -->
    <link href="{{ asset('build/assets/plugins/datatable/css/dataTables.bootstrap5.min.css') }}?v=<?php echo time(); ?>"
        rel="stylesheet" />
    <link href="{{ asset('build/assets/plugins/datatable/responsive.bootstrap5.css') }}?v=<?php echo time(); ?>"
        rel="stylesheet" />

    <!-- INTERNAL Sweet-Alert css -->
    <link href="{{ asset('build/assets/plugins/sweet-alert/sweetalert.css') }}?v=<?php echo time(); ?>" rel="stylesheet" />
@endsection

@section('content')
    <!--Page header-->
    <div class="page-header d-xl-flex d-block">
        <div class="page-leftheader">
            <h4 class="page-title"><span class="font-weight-normal text-muted ms-2">{{ lang('Department', 'menu') }}</span>
            </h4>
        </div>
    </div>
    <!--End Page header-->

    <div class="col-xl-12 col-lg-12 col-md-12">
        <div class="card ">
            <div class="card-header border-0 d-sm-max-flex">
                <h4 class="card-title">{{ lang('Department', 'menu') }}</h4>
                <div class="card-options mt-sm-max-2">
                    @can('Department Create')
                        <a href="javascript:void(0)" class="btn btn-secondary me-3"
                            id="create-new-department">{{ lang('Add Department') }}</a>
                    @endcan

                </div>
            </div>
            <div class="card-body">
                <div class="table-responsive spruko-delete" style="min-height: 230px;">
                    @can('Department Delete')
                        <button id="massdeletenotify" class="btn btn-outline-light btn-sm mb-4 data-table-btn"
                            style="display: none;"><i class="fe fe-trash"></i> {{ lang('Delete') }}</button>
                    @endcan

                    <table class="table table-bordered border-bottom text-nowrap ticketdeleterow w-100"
                        id="support-articlelists">
                        <thead>
                            <tr>
                                <th width="10">{{ lang('Sl.No') }}</th>
                                @can('Department Delete')
                                    <th width="10">
                                        <input type="checkbox" id="customCheckAll">
                                        <label for="customCheckAll"></label>
                                    </th>
                                @endcan
                                @cannot('Department Delete')
                                    <th width="10">
                                        <input type="checkbox" id="customCheckAll" disabled>
                                        <label for="customCheckAll"></label>
                                    </th>
                                @endcannot

                                <th>{{ lang('Department name') }}</th>
                                <th class="w-5">{{ lang('Status') }}</th>
                                <th class="w-5">{{ lang('Actions') }}</th>
                            </tr>
                        </thead>
                        <div id="hideAfterloading"><img src="{{ asset('build/assets/images/loader.svg') }}" alt="">
                            <p>loading.. Please Wait</p>
                        </div>
                    </table>
                </div>
            </div>
        </div>
    </div>
@endsection


@section('scripts')
    <!-- INTERNAL Vertical-scroll js-->
    <script src="{{ asset('build/assets/plugins/vertical-scroll/jquery.bootstrap.newsbox.js') }}"></script>

    <!-- INTERNAL Data tables -->
    <script src="{{ asset('build/assets/plugins/datatable/js/jquery.dataTables.min.js') }}"></script>
    <script src="{{ asset('build/assets/plugins/datatable/js/dataTables.bootstrap5.js') }}"></script>
    <script src="{{ asset('build/assets/plugins/datatable/dataTables.responsive.min.js') }}"></script>
    <script src="{{ asset('build/assets/plugins/datatable/responsive.bootstrap5.min.js') }}"></script>

    <!-- INTERNAL Sweet-Alert js-->
    <script src="{{ asset('build/assets/plugins/sweet-alert/sweetalert.min.js') }}"></script>
    {{-- @include('includes.admin.datatable') --}}

    <script type="text/javascript">
        document.addEventListener("DOMContentLoaded", (event) => {

            let tableData = @json($departments);
            let delPerm = @json(Auth::user()->can('Department Delete'));
            let editPerm = @json(Auth::user()->can('Department Edit'));
            let dateFormat = @json(setting('date_format'));
            var dataTable;
            var tableDropDown;

            function truncateTitle(title) {
                const maxLength = 40;
                return title.length > maxLength ? title.substring(0, maxLength) + '...' : title;
            }

            loadTable(tableData);

            function loadTable(tableData) {
                document.getElementById('hideAfterloading')?.classList.remove('d-none');
                document.getElementById('theadShow')?.classList.add('d-none');
                if (tableDropDown)
                    tableDropDown.destroy();

                // if (dataTable)
                //     dataTable.destroy();

                if ($.fn.DataTable.isDataTable('#support-articlelists')) {
                    $('#support-articlelists').DataTable().clear().destroy(); 
                }

                if (document.getElementById('tableBody'))
                    document.getElementById('tableBody').remove();

                let createTbody = document.createElement('tbody');
                createTbody.setAttribute('id', "tableBody");
                let rows = tableData.map((data, index) => {

                    return `
                        <tr>
                            <td>${index + 1}</td>
                            <td class="text-center">
                                ${delPerm
                                    ? ` <input type="checkbox" name="spruko_checkbox[]" class="checkall" value="${data.id}" />` : '~'
                                }
                            </td>

                            <td>${truncateTitle(data.departmentname)}</td>

                            <td>
                                ${ editPerm ? ( data.status == '1' ? `<label class="custom-switch form-switch mb-0">
                                                                                <input type="checkbox" name="status" data-id="${data.id}" id="myonoffswitch${data.id}" value="1" class="custom-switch-input tswitch" checked>
                                                                                <span class="custom-switch-indicator"></span>
                                                                            </label>` : `<label class="custom-switch form-switch mb-0">
                                                                                <input type="checkbox" name="status" data-id="${data.id}" id="myonoffswitch${data.id}" value="1" class="custom-switch-input tswitch">
                                                                                <span class="custom-switch-indicator"></span>
                                                                            </label>` ) : '~' }    
                            </td>

                            <td>
                                <div class="btn-list">
                                    ${editPerm ? ` <a href="javascript:void(0)" data-id="${data.id}"  class="action-btns1 edit-testimonial">
                                                                                <i class="feather feather-edit text-primary" data-id="${data.id} " data-bs-toggle="tooltip" data-bs-placement="top" title="{{ lang('Edit') }}"></i>
                                                                            </a>` : '~' 
                                    }


                                    ${delPerm  ? `  <a href="javascript:void(0)" data-id="${data.id}" class="action-btns1 delete-category" >
                                                                                <i class="feather feather-trash-2 text-danger" data-id="${data.id} " data-bs-toggle="tooltip" data-bs-placement="top" title="{{ lang('Delete') }}"></i>
                                                                            </a>` : '~' 
                                    }

                                </div>
                            </td>

                        </tr>
                    `;
                });

                createTbody.innerHTML = rows.length ? rows.join('') : '';
                document.getElementById('support-articlelists').appendChild(createTbody);
                let prev = {!! json_encode(lang('Previous')) !!};
                let next = {!! json_encode(lang('Next')) !!};
                let nodata = {!! json_encode(lang('No data available in table')) !!};
                let noentries = {!! json_encode(lang('No entries to show')) !!};
                let showing = {!! json_encode(lang('showing page')) !!};
                let ofval = {!! json_encode(lang('of')) !!};
                let maxRecordfilter = {!! json_encode(lang('- filtered from ')) !!};
                let maxRecords = {!! json_encode(lang('records')) !!};
                let entries = {!! json_encode(lang('entries')) !!};
                let show = {!! json_encode(lang('Show')) !!};
                let search = {!! json_encode(lang('Search...')) !!};
                // Reinitialize DataTable
                dataTable = $('#support-articlelists').DataTable({
                    language: {
                        searchPlaceholder: search,
                        scrollX: "100%",
                        sSearch: '',
                        paginate: {
                            previous: prev,
                            next: next
                        },
                        emptyTable: nodata,
                        infoFiltered: `${maxRecordfilter} _MAX_ ${maxRecords}`,
                        info: `${showing} _PAGE_ ${ofval} _PAGES_`,
                        infoEmpty: noentries,
                        lengthMenu: `${show} _MENU_ ${entries}`,
                    },
                    order: [],
                    columnDefs: [{
                        "orderable": false,
                        "targets": [0, 1, 3]
                    }],
                });

                document.querySelector('.card-body').style.setProperty('filter','')

                $('.form-select').select2({
                    minimumResultsForSearch: Infinity,
                    width: '100%'
                });
                $('.form-select').on('focus', function() {
                    $(this).attr('aria-hidden', 'false');
                });
                document.getElementById('hideAfterloading')?.classList.add('d-none');
                document.getElementById('theadShow')?.classList.remove('d-none');

                initializeTooltips();
                // checkboxNoLabel()
                // buttonAlertFunctions();
                // Pagenate();

                function initializeTooltips() {
                    var tooltipElements = document.querySelectorAll('[data-bs-toggle="tooltip"]');
                    tooltipElements.forEach(function(element) {
                        new bootstrap.Tooltip(element);
                    });
                }   
            }

            $(function() {
                "use strict";

                (function($) {

                    // Variables
                    var SITEURL = '{{ url('') }}';

                    // Csrf Field
                    $.ajaxSetup({
                        headers: {
                            'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                        }
                    });


                    //Mass Delete
                    $('body').on('click', '#massdeletenotify', function() {
                        var id = [];
                        var parent = [];
                        $('.checkall:checked').each(function() {
                            id.push($(this).val());
                            parent.push($(this).closest('tr'));
                        });
                        if (id.length > 0) {
                            swal({
                                    title: `{{ lang('Are you sure you want to continue?', 'alerts') }}`,
                                    text: "{{ lang('This might erase your records permanently', 'alerts') }}",
                                    icon: "warning",
                                    buttons: true,
                                    dangerMode: true,
                                })
                                .then((willDelete) => {
                                    if (willDelete) {
                                        $.ajax({
                                            url: "{{ route('department.deleteall') }}",
                                            method: "post",
                                            data: {
                                                id: id
                                            },
                                            success: function(data) {
                                                toastr.success(data.success);
                                                parent.forEach(element => {
                                                    element.addClass(
                                                        'fade-out');
                                                });

                                                let newData = tableData.filter(
                                                    item => !id.includes(
                                                        String(item.id)));
                                                tableData = newData;
                                                document.getElementById('hideAfterloading')?.classList.remove('d-none');
                                                document.querySelector('.card-body').style.setProperty('filter','blur(1px)');

                                                setTimeout(() => {
                                                    loadTable(
                                                        tableData);
                                                }, 1500);
                                                $('.checkall:checked').prop('checked', false);
                                                $('#customCheckAll').prop('checked', false);
                                                $('#massdelete').hide();
                                                // location.reload();

                                            },
                                            error: function(data) {
                                                console.log(data);
                                            }
                                        });
                                    }
                                });
                        } else {
                            toastr.error(
                                '{{ lang('Please select at least one check box.', 'alerts') }}'
                            );
                        }

                    });
                    //Mass Delete

                    $('.form-select').select2({
                        minimumResultsForSearch: Infinity,
                        width: '100%'
                    });

                    //checkbox script
                    $(document).ready(function() {

                        $(document).on('click', '#customCheckAll', function() {
                            $('.checkall').prop('checked', this.checked);
                            updateMassDeleteVisibility();
                        });

                        $(document).on('click', '.checkall', function() {
                            updateCustomCheckAll();
                            updateMassDeleteVisibility();
                        });


                        $(document).on('click', '.pagination a', function() {

                            setTimeout(function() {
                                updateMassDeleteVisibility();
                            }, 100);
                        });


                        $('#customCheckAll').prop('checked', false);


                        function updateMassDeleteVisibility() {
                            if ($('.checkall:checked').length === 0) {
                                $('#massdeletenotify').hide();
                            } else {
                                $('#massdeletenotify').show();
                            }
                        }


                        function updateCustomCheckAll() {
                            var totalCheckboxes = $('.checkall').length;
                            var checkedCheckboxes = $('.checkall:checked').length;

                            if (checkedCheckboxes === totalCheckboxes) {
                                $('#customCheckAll').prop('checked', true);
                            } else {
                                $('#customCheckAll').prop('checked', false);
                            }
                        }
                    });


                    // Status change department
                    $('body').on('click', '.tswitch', function() {
                        var _id = $(this).data("id");
                        var status = $(this).prop('checked') == true ? '1' : '0';
                        $.ajax({
                            type: "post",
                            url: SITEURL + "/admin/department/status" + '/' + _id,
                            data: {
                                'status': status
                            },
                            success: function(data) {
                                toastr.success(data.success);
                            },
                            error: function(data) {
                                console.log('Error:', data);
                            }
                        });
                    });




                    // Add department
                    $('#create-new-department').on('click', function() {
                        $("#department_id").val('');
                        $(".modal-title").text('{{ lang('Add New Department') }}');
                        $('#department_form').trigger("reset");
                        $('#adddepartment').modal('show');
                    });

                    // edit department
                    $('body').on('click', '.edit-testimonial', function() {
                        var id = $(this).data("id");
                        let _url = `{{ url('/admin/department/edit/${id}') }}`;
                        $('#departmentnameError').text('');

                        $(".modal-title").text('{{ lang('Edit department') }}');
                        $.ajax({
                            url: _url,
                            type: "GET",
                            success: function(response) {
                                if (response) {
                                    $('#departmentnameError').text('');
                                    $('#answerError').text('');
                                    $("#department_id").val(response.id);
                                    $("#departmentname").val(response
                                        .departmentname);
                                    if (response.status == "1") {
                                        $('#status').prop('checked', true);
                                    }

                                }
                                $('#adddepartment').modal('show');
                            }
                        });
                    });

                    // Delete department
                    $('body').on('click', '.delete-category', function() {
                        var id = $(this).data("id");
                        let parent = $(this).closest('tr');
                        let _url = `{{ url('/admin/department/delete/${id}') }}`;
                        let _token = $('meta[name="csrf-token"]').attr('content');
                        swal({
                                title: `{{ lang('Are you sure you want to continue?', 'alerts') }}`,
                                text: "{{ lang('This might erase your records permanently', 'alerts') }}",
                                icon: "warning",
                                buttons: true,
                                dangerMode: true,
                            })
                            .then((willDelete) => {
                                if (willDelete) {
                                    $.ajax({
                                        url: _url,
                                        type: 'DELETE',
                                        data: {
                                            _token: _token
                                        },
                                        success: function(response) {
                                            toastr.success(response.success);
                                            $(parent[0]).addClass('fade-out');
                                            let newData = tableData.filter(
                                                item => item
                                                .id != parseInt(id));
                                            tableData = newData;
                                            document.getElementById('hideAfterloading')?.classList.remove('d-none');
                                            document.querySelector('.card-body').style.setProperty('filter','blur(1px)');

                                            setTimeout(() => {
                                                parent.remove();
                                                loadTable(tableData);
                                            }, 1500);
                                            // location.reload();
                                        },
                                        error: function(data) {
                                            console.log('Error:', data);
                                        }
                                    });
                                }
                            });
                    });

                    // create the department
                    $('body').on('submit', '#department_form', function(e) {                      
                        e.preventDefault();                      
                        $('#departmentnameError').text('');
                        $('#answerError').text('');
                        var departmentname = $('#departmentname').val();
                        var status = $('#status').prop('checked') == true ? '1' : '0';

                        var id = $('#department_id').val();
                        var actionType = $('#btnsave').val();
                        var fewSeconds = 2;
                        $('#btnsave').html('Sending ... <i class="fa fa-spinner fa-spin"></i>');
                        $('#btnsave').prop('disabled', true);
                        setTimeout(function() {
                            $('#btnsave').prop('disabled', false);
                        }, fewSeconds * 1000);
                        let _url = `{{ route('department.create') }}`;
                        let _token = $('meta[name="csrf-token"]').attr('content');
                        $.ajax({
                            url: _url,
                            type: "POST",
                            data: {
                                id: id,
                                departmentname: departmentname,
                                status: status,
                                _token: _token
                            },
                            success: function(response) {
                                $('#btnsave').html('{{ lang('Save Changes') }}');
                                if (response.code == 200) {
                                    $('#departmentnameError').text('');
                                    $('#department_form').trigger("reset");
                                    $('#adddepartment').modal('hide');
                                    toastr.success(response.success);
                                    let newDept = response.data
                                    let exists = false;                    
                                    tableData = tableData.map(category => {if (category.id === newDept.id) {
                                            exists = true;
                                            return newDept;
                                        }
                                        return category;
                                    });

                                    if (!exists) {
                                        tableData.push(newDept);
                                    }
                                    document.getElementById('hideAfterloading')?.classList.remove('d-none');
                                    document.querySelector('.card-body').style.setProperty('filter','blur(1px)');

                                    loadTable(tableData);
                                    // location.reload();

                                }
                            },
                            error: function(response) {
                                $('#btnsave').html('{{ lang('Save Changes') }}');
                                $('#departmentnameError').text('');
                                $('#departmentnameError').text(response.responseJSON
                                    .errors.departmentname);

                            }
                        });

                    });

                    // cancel department
                    $('body').on('click', '#deptclose', function() {
                        $('#department_form').trigger("reset");
                    });


                })(jQuery);

            })
        });
    </script>
@endsection


@section('modal')
    @include('admin.department.model')
@endsection
